/* $Id: _chararraymodule.c,v 1.3 2002/03/11 21:28:34 jaytmiller Exp $ */

#include <stdio.h>
#include <math.h>
#include <signal.h>
#include <ctype.h>

#include <Python.h>

#include "libnumarray.h"

static PyObject *_Error;

/* returns length of string 's', or 'maxlen', whichever is smaller. */
int strnlen(char *s, int maxlen)
{
	int i;
	for(i=0; i<maxlen; i++)
		if (s[i] == 0)
			return i;
	return maxlen;
}

#if !defined(min)
static int min(int x, int y)
{
	if (x < y)
		return x;
	else
		return y;
}
#endif

#if !defined(max)
static int max(int x, int y)
{
	if (x > y)
		return x;
	else
		return y;
}
#endif

/* mystrdup duplicates the given string 's', copying up to 'size' bytes.
   This still works when 's' is not terminated within 'size' bytes. 
   Also uses python memory allocation.
*/
static char *mystrdup(char *s, int size)
{
	char *u = PyMem_Malloc(size), *v = u;
	if (!u)
		return (char *) PyErr_Format(_Error,
					     "mycat: Error allocating memory.");
	while(size-- && (*u++ = *s++));
	return v;
}

/* rstripw strips whitespace from the right side of 's', which can be up to
'n' characters long.  rstripw interprets 0 as terminating 's'.
rstripw never strips a string down to length 0; the last whitespace is always
spared.
*/
static void rstripw(char *s, int n)
{
	int i;
	for(i=strnlen(s,n)-1; i>=1; i--)  /* Never strip to length 0. */
	{
		int c = s[i];
		if (!c || isspace(c))
			s[i] = 0;
		else
			break;
	}
}

/* padw pads string 's' with blanks, for a total length of 'n' characters. */
static void padw(char *s, int n)
{
	int i;
	for(i=strnlen(s, n); i<n; i++)
		s[i] = ' ';
}

/* Strip executes rstripw on a python string. */
static PyObject *Strip(PyObject *obj, PyObject *args)
{
	char *s, *t;
	int  slen;  

	if (!PyArg_ParseTuple(args, "s#", &s, &slen))
		return PyErr_Format(_Error, 
				    "Invalid parameter to Strip.");
	if ((t = mystrdup(s, slen)))
	{
		rstripw(t, slen);
		obj = Py_BuildValue("s#", t, strnlen(t, slen));
		PyMem_Free(t);
		return obj;
	}
	else
		return NULL;
}


/* Pad executes padw on a python string. */
static PyObject *Pad(PyObject *obj, PyObject *args)
{
	char *s, *t;
	int  slen, rsize;

	if (!PyArg_ParseTuple(args, "s#i", &s, &slen, &rsize))
		return PyErr_Format(_Error, 
				    "Invalid parameter to Pad.");
	rsize = max(slen, rsize);
	if ((t = mystrdup(s, rsize)))
	{
		padw(t, rsize);
		obj = Py_BuildValue("s#", t, strnlen(t, rsize));
		PyMem_Free(t);
		return obj;
	}
	else
		return NULL;
}

/* Concat concatenates two arrays (a,b) into a third array (r) using NULL
terminated string semantics.  The full chararray concatenation requires
external stripping and padding.
*/
static PyObject  *Concat(PyObject *obj, PyObject *args)
{
	PyObject   *ao, *bo, *ro;
	NDInfo     ai, bi, ri;
	int        i, raw;
	char       *a, *b, *r;
  
	if (!PyArg_ParseTuple(args, "OOOi", &ao, &bo, &ro, &raw))
		return PyErr_Format(_Error, 
				    "Concat: Invalid parameters");
  
	if (getNDInfo( ao, &ai) || getNDInfo( bo, &bi) || getNDInfo( ro, &ri))
		return PyErr_Format(_Error, 
				    "Concat: error getting array info.");

	if (ai.itemsize + bi.itemsize > ri.itemsize)
		return PyErr_Format(_Error, 
				    "Concat: Invalid size parameters.");
  
	if (!ri.writeable)
		return PyErr_Format(_Error, 
				    "Concat: result array not writeable");

	a = (char *) NA_OFFSETDATA(ai);
	b = (char *) NA_OFFSETDATA(bi);
	r = (char *) NA_OFFSETDATA(ri);

	for(i=0; i<ai.nelem; i++, r+=ri.bytestride, a+=ai.bytestride, b+=bi.bytestride)
	{
		if (raw)
		{
			memcpy(r, a, ai.itemsize);
			memcpy(r+ai.itemsize, b, bi.itemsize);
		}
		else
		{
			int rused, bused;
			strncpy(r, a, ai.itemsize);
			rused = strnlen(r, ai.itemsize);
			bused = min(strnlen(b, bi.itemsize), ri.itemsize-rused);
			strncpy(r+rused, b, bused);
			memset(r+rused+bused, 0, ri.itemsize-rused-bused);
		}
	}
	return Py_BuildValue("O", ro);
}

/* StrCmp compares two CharArrays (a,b) into a third Int8 array (r) using 
   CharArray style stripping (rstripw) and strncmp.
*/
static PyObject  *StrCmp(PyObject *obj, PyObject *args)
{
	PyObject   *ao, *bo, *ro;
	NDInfo     ai, bi, ri;
	long       raw;
	int        i, nc;
	char       *a, *b, *r;
  
	if (!PyArg_ParseTuple(args, "OOOi", &ao, &bo, &ro, &raw))
		return PyErr_Format(_Error, 
				    "Concat: Invalid parameters");
  
	if (getNDInfo( ao, &ai) || getNDInfo( bo, &bi) || getNDInfo( ro, &ri))
		return PyErr_Format(_Error, 
				    "Concat: error getting array info.");

	if (!ri.writeable)
		return PyErr_Format(_Error, 
				    "Concat: result array not writeable");

	a = (char *) NA_OFFSETDATA(ai);
	b = (char *) NA_OFFSETDATA(bi);
	r = (char *) NA_OFFSETDATA(ri);

	nc = min(ai.itemsize, bi.itemsize);
	for(i=0; i<ai.nelem; i++, r+=ri.bytestride, a+=ai.bytestride, b+=bi.bytestride)
	{
		if (raw)
			*r = memcmp(a, b, nc);
		else
			*r = strncmp(a, b, nc);
	}
	return Py_BuildValue("O", ro);
}

/* StripAll applies rstripw to every element of a CharArray.  This can be used
to normalize chararrays initialized from a binary source.
*/
static PyObject  *StripAll(PyObject *obj, PyObject *args)
{
	int        i;
	PyObject   *ao;
	NDInfo     ai;
	char       *a;
  
	if (!PyArg_ParseTuple(args, "O", &ao))
		return PyErr_Format(_Error, 
				    "StripAll: Invalid parameters.");
	if (getNDInfo(ao, &ai))
		return PyErr_Format(_Error, 
				    "StripAll: error getting array info.");
	if (!ai.writeable)
		return PyErr_Format(_Error, 
				    "StripAll: result array not writeable.");
	a = (char *) NA_OFFSETDATA(ai);

	for(i=0; i<ai.nelem; i++, a+=ai.bytestride)
		rstripw(a, ai.itemsize);

	return Py_BuildValue("O", ao);
}

/* PadAll applies pad to every element of a CharArray.  This can be used
to normalize chararrays initialized from a binary source.
*/
static PyObject  *PadAll(PyObject *obj, PyObject *args)
{
	int        i;
	PyObject   *ao;
	NDInfo     ai;
	char       *a;
  
	if (!PyArg_ParseTuple(args, "O", &ao))
		return PyErr_Format(_Error, 
				    "PadAll: Invalid parameters.");
	if (getNDInfo(ao, &ai))
		return PyErr_Format(_Error, 
				    "PadAll: error getting array info.");
	if (!ai.writeable)
		return PyErr_Format(_Error, 
				    "PadAll: result array not writeable.");
	a = (char *) NA_OFFSETDATA(ai);

	for(i=0; i<ai.nelem; i++, a+=ai.bytestride)
		padw(a, ai.itemsize);

	return Py_BuildValue("O", ao);
}

/* MaxLen computes the maximum length of all stripped elements in a CharArray.
This information can be used to reduce the itemsize of the array.
*/
static PyObject  *MaxLen(PyObject *obj, PyObject *args)
{
	int        i, len=0;
	PyObject   *ao;
	NDInfo     ai;
	char       *a;
  
	if (!PyArg_ParseTuple(args, "O", &ao))
		return PyErr_Format(_Error, 
				    "MaxLen: Invalid parameters.");

	if (getNDInfo(ao, &ai))
		return PyErr_Format(_Error, 
				    "MaxLen: error getting array info.");
	a = (char *) NA_OFFSETDATA(ai);

	for(i=0; i<ai.nelem; i++, a+=ai.bytestride)
	{
		rstripw(a, ai.itemsize);
		len = max(len, strnlen(a, ai.itemsize));
	}

	return Py_BuildValue("i", len);
}

static PyObject *Eval(PyObject *obj, PyObject *args)
{
	PyObject *chararr, *numarr;
	NDInfo   ai, ni;
	char *a, *n;
	int i;
	PyObject *globals=PyEval_GetGlobals(), *locals=globals;
	PyObject *evalfunc = 	/* Must use system globals here. */
		PyRun_String("eval", Py_eval_input, globals, globals);
	if (!evalfunc)
		return PyErr_Format(_Error, 
				    "Eval: Failed fetching eval builtin.");
	if (!PyArg_ParseTuple(args, "OOOO", 
			      &chararr, &numarr, &globals, &locals))
		return PyErr_Format(_Error, 
				    "Eval: Invalid parameters.");
	if (getNDInfo(chararr, &ai))
		return PyErr_Format(_Error,
				    "Eval: error getting chararr info.");
	if (getNDInfo(numarr, &ni))
		return PyErr_Format(_Error,
				    "Eval: error getting numarr info.");
	a = NA_OFFSETDATA(ai);
	n = NA_OFFSETDATA(ni);

	for(i=0; i<ai.nelem; i++, a += ai.bytestride, n += ni.bytestride) {
		PyObject *astr, *aeval;
		astr = PyString_FromStringAndSize(a, strnlen(a, ai.itemsize));
		if (!astr)
			return PyErr_Format(_Error,
				"Eval: error creating item string.");
		aeval = PyObject_CallFunction(evalfunc, 
					      "(OOO)", astr, globals, locals);
		if (!aeval)
			return PyErr_Format(_Error,
				"Eval: error evaluating string[%d].", i);
		if (PyFloat_Check(aeval))
			*((double *) n) = PyFloat_AsDouble(aeval);
		else if (PyInt_Check(aeval))
			*((double *) n) = PyInt_AsLong(aeval);
		else if (PyLong_Check(aeval))
			*((double *) n) = PyLong_AsDouble(aeval);
		else
			return PyErr_Format(_Error,
				"Eval: non-numeric result for string[%d]", i);
		Py_DECREF(astr);
		Py_DECREF(aeval);
	}
	Py_DECREF(evalfunc);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject *Format(PyObject *obj, PyObject *args)
{
	PyObject *chararr, *format, *numarr;
	NDInfo   ai, ni;
	char *a, *n;
	int i;

	if (!PyArg_ParseTuple(args, "OSO", &chararr, &format, &numarr))
		return PyErr_Format(_Error, 
				    "Format: Invalid parameters.");
	if (getNDInfo(chararr, &ai))
		return PyErr_Format(_Error,
				    "Format: error getting chararr info.");
	if (getNDInfo(numarr, &ni))
		return PyErr_Format(_Error,
				    "Format: error getting numarr info.");

	a = NA_OFFSETDATA(ai);
	n = NA_OFFSETDATA(ni);

	for(i=0; i<ai.nelem; i++, a += ai.bytestride, n += ni.bytestride) {
		double d = *((double *) n);
		PyObject *args, *astr;
		if (!(args = Py_BuildValue("(d)", d)))
			return PyErr_Format(_Error, 
			    "Format: error building args tuple[%d].", i);
		if (!(astr = PyString_Format(format, args)))
			return PyErr_Format(_Error,
			    "Format: error formatting string[%d].", i);
		strncpy(a, PyString_AsString(astr), ai.itemsize);
		Py_DECREF(astr);
		Py_DECREF(args);
	}

	Py_INCREF(Py_None);
	return Py_None;
}

static PyMethodDef _chararrayMethods[] = {
	{"Concat", Concat, METH_VARARGS},
	{"Strip",  Strip, METH_VARARGS},
	{"Pad",    Pad, METH_VARARGS},
	{"StripAll",  StripAll, METH_VARARGS},
	{"PadAll",  PadAll, METH_VARARGS},
	{"MaxLen",  MaxLen, METH_VARARGS},
	{"StrCmp", StrCmp, METH_VARARGS},
	{"Eval",  Eval, METH_VARARGS},
	{"Format",  Format, METH_VARARGS},
	{NULL,     NULL}        /* Sentinel */
};

/* platform independent*/
#ifdef MS_WIN32
__declspec(dllexport)
#endif

	void init_chararray(void) {
	PyObject *m, *d;
	m = Py_InitModule("_chararray", _chararrayMethods);
	d = PyModule_GetDict(m);
	_Error = PyErr_NewException("_chararray.error", NULL, NULL);
	PyDict_SetItemString(d, "error", _Error);

	import_libnumarray();
}
