/**
 *
 * OpenGL Module for Python
 * 
 * Version: 0.9
 *
 * Date:    1998/01/15
 *
 * Authors: Tom Schwaller     <tom.schwaller@linux-magazin.de>
 *          Jim Hugunin       <hugunin@python.org>
 *          David Ascher      <da@skivs.ski.org>
 *          Michel Sanner     <sanner@scripps.edu>
 * 
 * Notes:
 *
 *   - the file glconst.py is also needed, as it defines all kinds of
 *     useful constants
 *     
 *   - If you do not have Numeric python installed, undefine NUMERIC
 *   - We have included the glut shapes for convenience. If you do not 
 *     want to use them undefine the GLUT constant, otherwise add 
 *     glut_shapes.c in the setup file like:
 *     opengl openglmodule.c glut_shapes.c -lGL -lGLU
 *
 *   - thanks to Mike Hartshorn <mjh@yorvic.york.ac.uk> for quite 
 *     some bug reports.
 *
 *   - Some new stuff: Hope to have more from users of this module
 *
 *     1996/08/30: gl_TranslateScene, gl_RotateScene
 *     1996/09/01: gl_DistFromLine (from Mike)
 * 
 *     1997/03/02: gl_SaveTiff from Lothar Birk <lb@ocean.fb12.TU-Berlin.DE>
 *                 use #define LIBTIFF 1 or -DLIBTIFF on the comand line
 *                 and -ltiff as axtra library
 *
 *     1997/05/12: gl_SavePPM fixed by Tom Schwaller 
 *     1997/05/12: gl_SaveEPS added by Tom Schwaller 
 *                 code from Togl1.3, was contributed by
 *                 Miguel A. De Riera Pasenau <miguel@DALILA.UPC.ES>
 *
 *     1998/01/16: moved all non-OpenGL spec stuff into openglutil.c
 *                 [DA & MS]
 *
 *     
***/

#define GLUT	1

#include "Python.h"
#ifdef MS_WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include <math.h>

#undef NUMERIC

static int typecode2gltype[] = {-1, GL_UNSIGNED_BYTE, GL_BYTE, GL_SHORT, GL_INT, -1, GL_FLOAT, -1, -1, -1, -1, -1};

static PyObject *gl_Error;

static PyObject *ErrorReturn(char *message)
{
  PyErr_SetString(gl_Error,message);
  return NULL;
}
#define ASSERT(E,M) if(!(E)) return ErrorReturn(M)

#define TRY(E) if(! (E)) return NULL

#ifdef NUMERIC

#include "arrayobject.h"

#define PyArray_ClearMemory(op, pitems) Py_DECREF((op))

static int PyArray_AsDoubleArray(PyObject **op, GLdouble **pitems, int *pn)
{
    PyArrayObject *mp;
    mp = (PyArrayObject *) PyArray_ContiguousFromObject (*op, PyArray_DOUBLE, 0, 1);
    if (!mp) return 0;
    *pitems = (GLdouble*) (mp->data);
    *pn = mp->dimensions[0];
    *op = (PyObject*) mp;
    return 1;
}

static int PyArray_AsFloatArray(PyObject **op, GLfloat **pitems, int *pn)
{
    PyArrayObject *mp;
    mp = (PyArrayObject *) PyArray_ContiguousFromObject (*op, PyArray_FLOAT, 0, 1);
    if (!mp) return 0;
    *pitems = (GLfloat*) (mp->data);
    *pn = mp->dimensions[0];
    *op = (PyObject*) mp;
    return 1;
}

static int PyArray_AsIntArray(PyObject **op, GLint **pitems, int *pn)
{
    PyArrayObject *mp;
    mp = (PyArrayObject *) PyArray_ContiguousFromObject (*op, PyArray_INT, 0, 1);
    if (!mp) return 0;
    *pitems =(GLint*) (mp->data);
    *pn = mp->dimensions[0];
    *op = (PyObject*) mp;
    return 1;
}

static int PyArray_AsShortArray(PyObject **op, GLshort **pitems, int *pn)
{
    PyArrayObject *mp;
    mp = (PyArrayObject *) PyArray_ContiguousFromObject (*op, PyArray_SHORT, 0, 1);
    if (!mp) return 0;
    *pitems =(GLshort*) (mp->data);
    *pn = mp->dimensions[0];
    *op = (PyObject*) mp;
    return 1;
}

#else

#include "abstract.h"

#define PyArray_ClearMemory(op, pitems) PyMem_DEL(pitems)

static int PyArray_AsDoubleArray(PyObject **op, GLdouble **pitems, int *pn)
{
    GLdouble *items;
    PyObject *item;
    int n, i;
    if (!PySequence_Check(*op)) return 0;
    n = PySequence_Length(*op);
    items = PyMem_NEW(GLdouble, n);
    if (items == NULL) {
	PyErr_NoMemory();
	return 0;
    }
    for (i=0; i<n; i++) {
	if (item = PySequence_GetItem(*op, i)) {
	   items[i] = PyFloat_AsDouble(item);
	   Py_DECREF(item);
        }
        if(PyErr_Occurred()) return 0;   
    }
    *pitems = items;
    *pn = n;
    return 1;
}

static int PyArray_AsFloatArray(PyObject **op, GLfloat **pitems, int *pn)
{
    GLfloat *items;
    PyObject *item;
    int n, i;
    if (!PySequence_Check(*op)) return 0;
    n = PySequence_Length(*op);
    items = PyMem_NEW(GLfloat, n);
    if (items == NULL) {
	PyErr_NoMemory();
	return 0;
    }
    for (i=0; i<n; i++) {
	if(item = PySequence_GetItem(*op, i)) {
	   items[i] = PyFloat_AsDouble(item);
	   Py_DECREF(item);
        }
        if(PyErr_Occurred()) return 0;
    }
    *pitems = items;
    *pn = n;
    return 1;
}

static int PyArray_AsIntArray(PyObject **op, GLint **pitems, int *pn)
{
    GLint *items;
    PyObject *item;
    int n, i;
    if (!PySequence_Check(*op)) return 0;
    n = PySequence_Length(*op);
    items = PyMem_NEW(GLint, n);
    if (items == NULL) {
	PyErr_NoMemory();
	return 0;
    }
    for (i=0; i<n; i++) {
	if (item = PySequence_GetItem(*op, i)) {
	   items[i] = PyInt_AsLong(item);
	   Py_DECREF(item);
	}
        if(PyErr_Occurred()) return 0;    
    }
    *pitems = items;
    *pn = n;
    return 1;
}

static int PyArray_AsShortArray(PyObject **op, GLshort **pitems, int *pn)
{
    GLshort *items;
    PyObject *item;
    int n, i;
    if (!PySequence_Check(*op)) return 0;
    n = PySequence_Length(*op);
    items = PyMem_NEW(GLshort, n);
    if (items == NULL) {
	PyErr_NoMemory();
	return 0;
    }
    for (i = 0; i < n; i++) {
	if (item = PySequence_GetItem(*op, i)) {
	   items[i] = PyInt_AsLong(item);
	   Py_DECREF(item);
	}
        if(PyErr_Occurred()) return 0;
    }
    *pitems = items;
    *pn = n;
    return 1;

}

#endif /* Not NUMERIC */

/* 
########################################################################
*/

static PyObject* gl_Accum(PyObject* self, PyObject* args)
{
  	GLenum arg1;
  	GLfloat arg2;
	TRY (PyArg_ParseTuple(args, "if", &arg1, &arg2));
  	glAccum(arg1, arg2);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_AlphaFunc(PyObject* self, PyObject* args)
{
	GLenum arg1 ;
  	GLclampf arg2 ;
	TRY (PyArg_ParseTuple(args, "if", &arg1, &arg2));
  	glAlphaFunc(arg1, arg2);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Begin(PyObject* self, PyObject* args)
{
  	GLenum arg1 ;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glBegin(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_BlendFunc(PyObject* self, PyObject* args)
{
	GLenum arg1, arg2;
	TRY (PyArg_ParseTuple(args, "ii", &arg1, &arg2));
  	glBlendFunc(arg1 , arg2);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_CallList(PyObject* self, PyObject* args)
{
	GLuint	arg1;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glCallList(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Clear(PyObject* self, PyObject* args)
{
	GLbitfield arg1;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glClear(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_ClearAccum(PyObject* self, PyObject* args)
{
	GLfloat arg1, arg2, arg3, arg4;
	TRY (PyArg_ParseTuple(args, "ffff", &arg1, &arg2, &arg3, &arg4));
  	glClearAccum(arg1 , arg2 , arg3 , arg4);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_ClearColor(PyObject* self, PyObject* args)
{
	GLclampf arg1, arg2, arg3, arg4;
	TRY (PyArg_ParseTuple(args, "ffff", &arg1, &arg2, &arg3, &arg4));
  	glClearColor(arg1 , arg2 , arg3 , arg4);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_ClearDepth(PyObject* self, PyObject* args)
{
	GLclampd arg1;
	TRY (PyArg_ParseTuple(args, "d", &arg1));
  	glClearDepth(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_ClearIndex(PyObject* self, PyObject* args)
{
	GLfloat arg1;
	TRY (PyArg_ParseTuple(args, "f", &arg1));
  	glClearIndex(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_ClearStencil(PyObject* self, PyObject* args)
{
	GLint arg1;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glClearStencil(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Color3d(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, arg3, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "ddd", &arg1, &arg2, &arg3)) 	
  		glColor3d(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsDoubleArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glColor3dv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Color3f(PyObject* self, PyObject* args)
{
	GLfloat arg1, arg2, arg3, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "fff", &arg1, &arg2, &arg3)) 	
  		glColor3f(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsFloatArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glColor3fv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Color3i(PyObject* self, PyObject* args)
{
	GLint arg1, arg2, arg3, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "iii", &arg1, &arg2, &arg3)) 	
  		glColor3i(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsIntArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glColor3iv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Color3s(PyObject* self, PyObject* args)
{
	GLshort arg1, arg2, arg3, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "hhh", &arg1, &arg2, &arg3)) 	
  		glColor3s(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsShortArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glColor3sv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Color4d(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, arg3, arg4, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "dddd", &arg1, &arg2, &arg3, &arg4)) 	
  		glColor4d(arg1, arg2, arg3, arg4);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsDoubleArray(&op, &vert, &n));
		if (n < 4) {
			PyErr_SetString(gl_Error,  "need element with at least 4 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glColor4dv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Color4f(PyObject* self, PyObject* args)
{
	GLfloat arg1, arg2, arg3, arg4, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "ffff", &arg1, &arg2, &arg3, &arg4)) 	
  		glColor4f(arg1, arg2, arg3, arg4);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsFloatArray(&op, &vert, &n));
		if (n < 4) {
			PyErr_SetString(gl_Error,  "need element with at least 4 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glColor4fv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Color4i(PyObject* self, PyObject* args)
{
	GLint arg1, arg2, arg3, arg4, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "iiii", &arg1, &arg2, &arg3, &arg4)) 	
  		glColor4i(arg1, arg2, arg3, arg4);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsIntArray(&op, &vert, &n));
		if (n < 4) {
			PyErr_SetString(gl_Error,  "need element with at least 4 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glColor4iv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Color4s(PyObject* self, PyObject* args)
{
	GLshort arg1, arg2, arg3, arg4, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "hh", &arg1, &arg2, &arg3, &arg4)) 	
  		glColor4s(arg1, arg2, arg3, arg4);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsShortArray(&op, &vert, &n));
		if (n < 4) {
			PyErr_SetString(gl_Error,  "need element with at least 4 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glColor4sv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_ColorMask(PyObject* self, PyObject* args)
{
	GLboolean arg1, arg2, arg3, arg4;
	TRY (PyArg_ParseTuple(args, "bbbb", &arg1, &arg2, &arg3, &arg4));
  	glColorMask(arg1 , arg2 , arg3 , arg4);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_ColorMaterial(PyObject* self, PyObject* args)
{
	GLenum arg1, arg2;
	TRY (PyArg_ParseTuple(args, "ii", &arg1, &arg2));
  	glColorMaterial(arg1 , arg2);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_CopyPixels(PyObject* self, PyObject* args)
{
	GLint arg1, arg2;
 	GLsizei arg3, arg4;
	GLenum arg5 ;
	TRY (PyArg_ParseTuple(args, "iiiii", &arg1, &arg2, &arg3, &arg4, &arg5));
  	glCopyPixels(arg1 , arg2 , arg3 , arg4 , arg5);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_CullFace(PyObject* self, PyObject* args)
{
	GLenum arg1 ;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glCullFace(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_DeleteLists(PyObject* self, PyObject* args)
{
	GLuint arg1 ;
	GLsizei arg2 ;
	TRY (PyArg_ParseTuple(args, "ii", &arg1, &arg2));
  	glDeleteLists(arg1, arg2);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_DepthFunc(PyObject* self, PyObject* args)
{
	GLenum arg1 ;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glDepthFunc(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_DepthMask(PyObject* self, PyObject* args)
{
	GLboolean arg1;
	TRY (PyArg_ParseTuple(args, "b", &arg1));
  	glDepthMask(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_DepthRange(PyObject* self, PyObject* args)
{
	GLclampd arg1, arg2;
	TRY (PyArg_ParseTuple(args, "dd", &arg1, &arg2));
  	glDepthRange(arg1 , arg2);
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Disable(PyObject* self, PyObject* args)
{
	GLenum arg1 ;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glDisable(arg1);
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_DrawBuffer(PyObject* self, PyObject* args)
{
	GLenum arg1 ;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glDrawBuffer(arg1);
  	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_EdgeFlag(PyObject* self, PyObject* args)
{
	GLboolean arg1 ;
	TRY (PyArg_ParseTuple(args, "b", &arg1));
  	glEdgeFlag(arg1);
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Enable(PyObject* self, PyObject* args)
{
	GLenum arg1 ;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glEnable(arg1);
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_End(PyObject* self, PyObject* args)
{
	TRY (PyArg_ParseTuple(args, ""));
  	glEnd();
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_EndList(PyObject* self, PyObject* args)
{
	TRY (PyArg_ParseTuple(args, ""));
  	glEndList();
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_EvalCoord1d(PyObject* self, PyObject* args)
{
	GLdouble arg1 ;
	TRY (PyArg_ParseTuple(args, "d", &arg1));
  	glEvalCoord1d(arg1);
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_EvalCoord1f(PyObject* self, PyObject* args)
{
	GLfloat arg1 ;
	TRY (PyArg_ParseTuple(args, "f", &arg1));
  	glEvalCoord1f(arg1);
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_EvalCoord2d(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, *coords;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "dd", &arg1, &arg2)) 	
  		glEvalCoord2d(arg1, arg2);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsDoubleArray(&op, &coords, &n));
		if (n < 2) {
			PyErr_SetString(gl_Error,  "need element with at least 2 items");
			PyArray_ClearMemory(op, coords);
			return NULL;
		}
		glEvalCoord2dv(coords);
		PyArray_ClearMemory(op, coords);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_EvalCoord2f(PyObject* self, PyObject* args)
{
	GLfloat arg1, arg2, *coords;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "ff", &arg1, &arg2)) 	
  		glEvalCoord2f(arg1, arg2);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsFloatArray(&op, &coords, &n));
		if (n < 2) {
			PyErr_SetString(gl_Error,  "need element with at least 2 items");
			PyArray_ClearMemory(op, coords);
			return NULL;
		}
		glEvalCoord2fv(coords);
		PyArray_ClearMemory(op, coords);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_EvalMesh1(PyObject* self, PyObject* args)
{
	GLenum arg1 ;
	GLint arg2, arg3 ;
	TRY (PyArg_ParseTuple(args, "iii", &arg1, &arg2, &arg3));
  	glEvalMesh1(arg1, arg2, arg3);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_EvalMesh2(PyObject* self, PyObject* args)
{
	GLenum arg1;
	GLint arg2, arg3, arg4, arg5;
	TRY (PyArg_ParseTuple(args, "iiiii", &arg1, &arg2, &arg3, &arg4, &arg5));
  	glEvalMesh2(arg1, arg2, arg3, arg4, arg5);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_EvalPoint1(PyObject* self, PyObject* args)
{
	GLint arg1;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
	glEvalPoint1(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_EvalPoint2(PyObject* self, PyObject* args)
{
	GLint arg1, arg2;
	TRY (PyArg_ParseTuple(args, "ii", &arg1, &arg2));
	glEvalPoint2(arg1, arg2);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Finish(PyObject* self, PyObject* args)
{
	TRY (PyArg_ParseTuple(args, ""));
  	glFinish();
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Flush(PyObject* self, PyObject* args)
{
	TRY (PyArg_ParseTuple(args, ""));
  	glFlush();
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Fogf(PyObject* self, PyObject* args)
{
	GLenum arg1;
	GLfloat arg2, *rest;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "if", &arg1, &arg2))
  		glFogf(arg1, arg2);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "iO", &arg1, &op));
		TRY (PyArray_AsFloatArray(&op, &rest, &n));
		if ((n == 1) || (n > 3)) {
			glFogfv(arg1, rest);
			PyArray_ClearMemory(op, rest);
		}
		else {
			PyErr_SetString(gl_Error,  "second argument needs 1 or at least 4 items!");
			PyArray_ClearMemory(op, rest);
			return NULL;
		}
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Fogi(PyObject* self, PyObject* args)
{
	GLenum arg1;
	GLint arg2, *rest;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "ii", &arg1, &arg2))
  		glFogi(arg1, arg2);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "iO", &arg1, &op));
		TRY (PyArray_AsIntArray(&op, &rest, &n));
		if ((n == 1) || (n>3)) {
			glFogiv(arg1, rest);
			PyArray_ClearMemory(op, rest);
		}
		else {
			PyErr_SetString(gl_Error,  "second argument needs 1 or at least 4 items!");
			PyArray_ClearMemory(op, rest);
			return NULL;
		}
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_FrontFace(PyObject* self, PyObject* args)
{
	GLenum arg1;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glFrontFace(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Frustum(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, arg3, arg4, arg5, arg6;
	TRY (PyArg_ParseTuple(args, "dddddd", &arg1, &arg2, &arg3, &arg4, &arg5, &arg6));
  	glFrustum(arg1, arg2, arg3, arg4, arg5, arg6);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Hint(PyObject* self, PyObject* args)
{ 
	GLenum arg1, arg2;
	TRY (PyArg_ParseTuple(args, "ii", &arg1, &arg2));
  	glHint(arg1 , arg2);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_IndexMask(PyObject* self, PyObject* args)
{
	GLuint arg1;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glIndexMask(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Indexd(PyObject* self, PyObject* args)
{
	GLdouble arg1;
	TRY (PyArg_ParseTuple(args, "d", &arg1));
  	glIndexd(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Indexf(PyObject* self, PyObject* args)
{
	GLfloat arg1;
	TRY (PyArg_ParseTuple(args, "f", &arg1));
  	glIndexf(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Indexi(PyObject* self, PyObject* args)
{
	GLint arg1;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glIndexi(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Indexs(PyObject* self, PyObject* args)
{
	GLshort arg1;
	TRY (PyArg_ParseTuple(args, "h", &arg1));
  	glIndexs(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_InitNames(PyObject* self, PyObject* args)
{
	TRY (PyArg_ParseTuple(args, ""));
  	glInitNames();
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_LightModelf(PyObject* self, PyObject* args)
{	
	GLenum arg1;
	GLfloat arg2, *rest;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "if", &arg1, &arg2))
  		glLightModelf(arg1, arg2);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "iO", &arg1, &op));
		TRY (PyArray_AsFloatArray(&op, &rest, &n));
		if ((n == 1) || (n>3)) {
			glLightModelfv(arg1, rest);
			PyArray_ClearMemory(op, rest);
		}
		else {
			PyErr_SetString(gl_Error,  "second argument needs 1 or at least 4 items!");
			PyArray_ClearMemory(op, rest);
			return NULL;
		}
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_LightModeli(PyObject* self, PyObject* args)
{
	GLenum arg1;
	GLint arg2, *rest;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "ii", &arg1, &arg2))
  		glLightModeli(arg1, arg2);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "iO", &arg1, &op));
		TRY (PyArray_AsIntArray(&op, &rest, &n));
		if ((n == 1) || (n>3)) {
			glLightModeliv(arg1, rest);
			PyArray_ClearMemory(op, rest);
		}
		else {
			PyErr_SetString(gl_Error,  "second argument needs 1 or at least 4 items!");
			PyArray_ClearMemory(op, rest);
			return NULL;
		}
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Lightf(PyObject* self, PyObject* args)
{
	GLenum arg1, arg2;
	GLfloat arg3, *rest;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "iif", &arg1, &arg2, &arg3))
  		glLightf(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "iiO", &arg1, &arg2, &op));
		TRY (PyArray_AsFloatArray(&op, &rest, &n));
		if ((n == 1) || (n > 2)) {
			glLightfv(arg1, arg2, rest);
			PyArray_ClearMemory(op, rest);
		}
		else {
			PyErr_SetString(gl_Error,  "second argument needs 1 or at least 3 items!");
			PyArray_ClearMemory(op, rest);
			return NULL;
		}
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Lighti(PyObject* self, PyObject* args)
{
	GLenum arg1, arg2;
	GLint arg3, *rest;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "iii", &arg1, &arg2, &arg3))
  		glLighti(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "iiO", &arg1, &arg2, &op));
		TRY (PyArray_AsIntArray(&op, &rest, &n));
		if ((n == 1) || (n > 2)) {
			glLightiv(arg1, arg2, rest);
			PyArray_ClearMemory(op, rest);
		}
		else {
			PyErr_SetString(gl_Error,  "second argument needs 1 or at least 3 items!");
			PyArray_ClearMemory(op, rest);
			return NULL;
		}
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_LineStipple(PyObject* self, PyObject* args)
{
	GLint arg1;
	GLushort arg2;
	TRY (PyArg_ParseTuple(args, "ih", &arg1, &arg2));
  	glLineStipple(arg1, arg2);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_LineWidth(PyObject* self, PyObject* args)
{
	GLfloat arg1;
 	TRY (PyArg_ParseTuple(args, "f", &arg1));
  	glLineWidth(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_ListBase(PyObject* self, PyObject* args)
{
	GLuint arg1 ;
 	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glListBase(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_LoadIdentity(PyObject* self, PyObject* args)
{
	TRY (PyArg_ParseTuple(args, ""));
  	glLoadIdentity();
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_LoadName(PyObject* self, PyObject* args)
{
	GLuint arg1;
 	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glLoadName(arg1);
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_LogicOp(PyObject* self, PyObject* args)
{
	GLenum arg1;
 	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glLogicOp(arg1);
 	Py_INCREF(Py_None);
	return Py_None;
}


static PyObject* gl_Map1d(PyObject* self, PyObject* args)
{

  GLenum arg1;
  GLdouble arg2;
  GLdouble arg3;
  GLint arg4;
  GLint arg5;
  PyObject *arg6;
  GLdouble *points;
  int n;

  TRY (PyArg_ParseTuple(args, "iddiiO", &arg1, &arg2, &arg3, &arg4, &arg5, &arg6));
  TRY (PyArray_AsDoubleArray(&arg6, &points, &n));
  glMap1d(arg1, arg2, arg3, arg4, arg5, points);
  Py_DECREF(arg6);

  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject* gl_Map1f(PyObject* self, PyObject* args)
{
  GLenum arg1;
  GLfloat arg2;
  GLfloat arg3;
  GLint arg4;
  GLint arg5;
  PyObject *arg6;
  GLfloat *points;
  int n;

  TRY (PyArg_ParseTuple(args, "iffiiO", &arg1, &arg2, &arg3, &arg4, &arg5, &arg6));
  TRY (PyArray_AsFloatArray(&arg6, &points, &n));
  glMap1f(arg1, arg2, arg3, arg4, arg5, points);
  Py_DECREF(arg6);

  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject* gl_Map2d(PyObject* self, PyObject* args)
{
  GLenum arg1;
  GLdouble arg2;
  GLdouble arg3;
  GLint arg4;
  GLint arg5;
  GLdouble arg6;
  GLdouble arg7;
  GLint arg8;
  GLint arg9;
  PyObject *arg10;
  GLdouble *points;
  int n;

  TRY (PyArg_ParseTuple(args, "iddiiddiiO",
			&arg1, &arg2, &arg3, &arg4, &arg5,
			&arg6, &arg7, &arg8, &arg9, &arg10));

  TRY (PyArray_AsDoubleArray(&arg10, &points, &n));
  glMap2d(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, points);

  Py_DECREF(arg10);

  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject* gl_Map2f(PyObject* self, PyObject* args)
{
  GLenum arg1;
  GLfloat arg2;
  GLfloat arg3;
  GLint arg4;
  GLint arg5;
  GLfloat arg6;
  GLfloat arg7;
  GLint arg8;
  GLint arg9;
  PyObject *arg10;
  GLfloat *points;
  int n;

  TRY (PyArg_ParseTuple(args, "iffiiffiiO",
			&arg1, &arg2, &arg3, &arg4, &arg5,
			&arg6, &arg7, &arg8, &arg9, &arg10));

  TRY (PyArray_AsFloatArray(&arg10, &points, &n));
  glMap2f(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, points);

  Py_DECREF(arg10);

  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject* gl_MapGrid1d(PyObject* self, PyObject* args)
{
	GLint arg1;
	GLdouble arg2, arg3;
	TRY (PyArg_ParseTuple(args, "idd", &arg1, &arg2, &arg3));
  	glMapGrid1d(arg1, arg2 , arg3);
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_MapGrid1f(PyObject* self, PyObject* args)
{
	GLint arg1;
	GLfloat arg2, arg3;
	TRY (PyArg_ParseTuple(args, "iff", &arg1, &arg2, &arg3));
  	glMapGrid1f(arg1, arg2 , arg3);
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_MapGrid2d(PyObject* self, PyObject* args)
{
	GLint arg1, arg4;
	GLdouble arg2, arg3, arg5, arg6;
	TRY (PyArg_ParseTuple(args, "iddidd", &arg1, &arg2, &arg3, &arg4, &arg5, &arg6));
  	glMapGrid2d(arg1, arg2, arg3, arg4, arg5, arg6);
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_MapGrid2f(PyObject* self, PyObject* args)
{
	GLint arg1, arg4;
	GLfloat arg2, arg3, arg5, arg6;
	TRY (PyArg_ParseTuple(args, "iffiff", &arg1, &arg2, &arg3, &arg4, &arg5, &arg6));
  	glMapGrid2f(arg1, arg2, arg3, arg4, arg5, arg6);
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Materialf(PyObject* self, PyObject* args)
{
	GLenum arg1, arg2;
	GLfloat arg3, *rest;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "iif", &arg1, &arg2, &arg3))
  		glMaterialf(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "iiO", &arg1, &arg2, &op));
		TRY (PyArray_AsFloatArray(&op, &rest, &n));
		if ((n == 1) || (n>3)) {
			glMaterialfv(arg1, arg2, rest);
			PyArray_ClearMemory(op, rest);
		}
		else {
			PyErr_SetString(gl_Error,  "second argument needs 1 or at least 4 items!");
			PyArray_ClearMemory(op, rest);
			return NULL;
		}
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Materiali(PyObject* self, PyObject* args)
{
	GLenum arg1, arg2;
	GLint arg3, *rest;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "iii", &arg1, &arg2, &arg3))
  		glMateriali(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "iiO", &arg1, &arg2, &op));
		TRY (PyArray_AsIntArray(&op, &rest, &n));
		if ((n == 1) || (n > 3)) {
			glMaterialiv(arg1, arg2, rest);
			PyArray_ClearMemory(op, rest);
		}
		else {
			PyErr_SetString(gl_Error,  "second argument needs 1 or at least 4 items!");
			PyArray_ClearMemory(op, rest);
			return NULL;
		}
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_MatrixMode(PyObject* self, PyObject* args)
{
	GLenum arg1 ;
 	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glMatrixMode(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_MultMatrixd(PyObject* self, PyObject* args)
{
	GLdouble *mat;
	PyObject *op;
	int n;
 	TRY (PyArg_ParseTuple(args, "O", &op));
	TRY (PyArray_AsDoubleArray(&op, &mat, &n));
	if (n < 16) {
		PyErr_SetString(gl_Error, "need element with at least 16 items");
		PyArray_ClearMemory(op, mat);
		return NULL;
	}
	glMultMatrixd(mat);
	PyArray_ClearMemory(op, mat);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_MultMatrixf(PyObject* self, PyObject* args)
{
	GLfloat *mat;
    PyObject *op;
	int n;
 	TRY (PyArg_ParseTuple(args, "O", &op));
	TRY (PyArray_AsFloatArray(&op, &mat, &n));
    if (n < 16) {
		PyErr_SetString(gl_Error, "need element with at least 16 items");
		PyArray_ClearMemory(op, mat);
		return NULL;
	}
	glMultMatrixf(mat);
	PyArray_ClearMemory(op, mat);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_NewList(PyObject* self, PyObject* args)
{
	GLuint arg1;
	GLenum arg2;
 	TRY (PyArg_ParseTuple(args, "ii", &arg1, &arg2));
  	glNewList(arg1, arg2);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Normal3d(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, arg3, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "ddd", &arg1, &arg2, &arg3)) 	
  		glNormal3d(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsDoubleArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glNormal3dv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Normal3f(PyObject* self, PyObject* args)
{
	GLfloat arg1, arg2, arg3, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "fff", &arg1, &arg2, &arg3)) 	
  		glNormal3f(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsFloatArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glNormal3fv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Normal3i(PyObject* self, PyObject* args)
{
	GLint arg1, arg2, arg3, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "iii", &arg1, &arg2, &arg3)) 	
  		glNormal3i(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsIntArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glNormal3iv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Normal3s(PyObject* self, PyObject* args)
{
	GLshort arg1, arg2, arg3, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "hhh", &arg1, &arg2, &arg3)) 	
  		glNormal3s(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsShortArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glNormal3sv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Ortho(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, arg3, arg4, arg5, arg6;
	TRY (PyArg_ParseTuple(args, "dddddd", &arg1, &arg2, &arg3, &arg4, &arg5, &arg6));
  	glOrtho(arg1, arg2, arg3, arg4, arg5, arg6);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_PassThrough(PyObject* self, PyObject* args)
{
	GLfloat arg1;
	TRY (PyArg_ParseTuple(args, "f", &arg1));
	glPassThrough(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_PixelStoref(PyObject* self, PyObject* args)
{
	GLenum arg1;
  	GLfloat arg2;
	TRY (PyArg_ParseTuple(args, "if", &arg1, &arg2));
  	glPixelStoref(arg1, arg2);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_PixelStorei(PyObject* self, PyObject* args)
{
	GLenum arg1;
  	GLint arg2;
	TRY (PyArg_ParseTuple(args, "ii", &arg1, &arg2));
  	glPixelStorei(arg1, arg2);
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_PixelTransferf(PyObject* self, PyObject* args)
{
	GLenum arg1;
  	GLfloat arg2;
	TRY (PyArg_ParseTuple(args, "if", &arg1, &arg2));
  	glPixelTransferf(arg1, arg2);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_PixelTransferi(PyObject* self, PyObject* args)
{
	GLenum arg1;
  	GLint arg2;
	TRY (PyArg_ParseTuple(args, "ii", &arg1, &arg2));
  	glPixelTransferi(arg1, arg2);
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_PixelZoom(PyObject* self, PyObject* args)
{
	GLfloat arg1, arg2;
	TRY (PyArg_ParseTuple(args, "ff", &arg1, &arg2));
  	glPixelZoom(arg1, arg2);
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_PointSize(PyObject* self, PyObject* args)
{
	GLfloat arg1;
	TRY (PyArg_ParseTuple(args, "f", &arg1));
  	glPointSize(arg1);
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_PolygonMode(PyObject* self, PyObject* args)
{
	GLenum arg1, arg2;
	TRY (PyArg_ParseTuple(args, "ii", &arg1, &arg2));
  	glPolygonMode(arg1, arg2);
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_PopAttrib(PyObject* self, PyObject* args)
{
	TRY (PyArg_ParseTuple(args, ""));
  	glPopAttrib();
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_PopMatrix(PyObject* self, PyObject* args)
{
	TRY (PyArg_ParseTuple(args, ""));
  	glPopMatrix();
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_PopName(PyObject* self, PyObject* args)
{
	TRY (PyArg_ParseTuple(args, ""));
  	glPopName();
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_PushAttrib(PyObject* self, PyObject* args)
{
	GLbitfield arg1 ;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glPushAttrib(arg1);
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_PushMatrix(PyObject* self, PyObject* args)
{
	TRY (PyArg_ParseTuple(args, ""));
  	glPushMatrix();
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_PushName(PyObject* self, PyObject* args)
{
	GLuint arg1 ;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glPushName(arg1);
 	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_RasterPos2d(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "dd", &arg1, &arg2)) 	
  		glRasterPos2d(arg1, arg2);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsDoubleArray(&op, &vert, &n));
		if (n < 2) {
			PyErr_SetString(gl_Error,  "need element with at least 2 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glRasterPos2dv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_RasterPos2f(PyObject* self, PyObject* args)
{
	GLfloat arg1, arg2, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "ff", &arg1, &arg2)) 	
  		glRasterPos2f(arg1, arg2);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsFloatArray(&op, &vert, &n));
		if (n < 2) {
			PyErr_SetString(gl_Error,  "need element with at least 2 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glRasterPos2fv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_RasterPos2i(PyObject* self, PyObject* args)
{
	GLint arg1, arg2, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "ii", &arg1, &arg2)) 	
  		glRasterPos2i(arg1, arg2);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsIntArray(&op, &vert, &n));
		if (n < 2) {
			PyErr_SetString(gl_Error,  "need element with at least 2 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glRasterPos2iv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_RasterPos2s(PyObject* self, PyObject* args)
{
	GLshort arg1, arg2, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "hh", &arg1, &arg2)) 	
  		glRasterPos2s(arg1, arg2);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsShortArray(&op, &vert, &n));
		if (n < 2) {
			PyErr_SetString(gl_Error,  "need element with at least 2 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glRasterPos2sv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_RasterPos3d(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, arg3, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "ddd", &arg1, &arg2, &arg3)) 	
  		glRasterPos3d(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsDoubleArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glRasterPos3dv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_RasterPos3f(PyObject* self, PyObject* args)
{
	GLfloat arg1, arg2, arg3, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "fff", &arg1, &arg2, &arg3)) 	
  		glRasterPos3f(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsFloatArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glRasterPos3fv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_RasterPos3i(PyObject* self, PyObject* args)
{
	GLint arg1, arg2, arg3, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "iii", &arg1, &arg2, &arg3)) 	
  		glRasterPos3i(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsIntArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glRasterPos3iv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_RasterPos3s(PyObject* self, PyObject* args)
{
	GLshort arg1, arg2, arg3, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "hhh", &arg1, &arg2, &arg3)) 	
  		glRasterPos3s(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsShortArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glRasterPos3sv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_RasterPos4d(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, arg3, arg4, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "dddd", &arg1, &arg2, &arg3, &arg4)) 	
  		glRasterPos4d(arg1, arg2, arg3, arg4);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsDoubleArray(&op, &vert, &n));
		if (n < 4) {
			PyErr_SetString(gl_Error,  "need element with at least 4 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glRasterPos4dv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_RasterPos4f(PyObject* self, PyObject* args)
{
	GLfloat arg1, arg2, arg3, arg4, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "ffff", &arg1, &arg2, &arg3, &arg4)) 	
  		glRasterPos4f(arg1, arg2, arg3, arg4);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsFloatArray(&op, &vert, &n));
		if (n < 4) {
			PyErr_SetString(gl_Error,  "need element with at least 4 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glRasterPos4fv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_RasterPos4i(PyObject* self, PyObject* args)
{
	GLint arg1, arg2, arg3, arg4, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "iiii", &arg1, &arg2, &arg3, &arg4)) 	
  		glRasterPos4i(arg1, arg2, arg3, arg4);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsIntArray(&op, &vert, &n));
		if (n < 4) {
			PyErr_SetString(gl_Error,  "need element with at least 4 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glRasterPos4iv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_RasterPos4s(PyObject* self, PyObject* args)
{
	GLshort arg1, arg2, arg3, arg4, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "hhhh", &arg1, &arg2, &arg3, &arg4)) 	
  		glRasterPos4s(arg1, arg2, arg3, arg4);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsShortArray(&op, &vert, &n));
		if (n < 4) {
			PyErr_SetString(gl_Error,  "need element with at least 4 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glRasterPos4sv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_ReadBuffer(PyObject* self, PyObject* args)
{
	GLenum arg1 ;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glReadBuffer(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Rectd(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, arg3, arg4, *vert1, *vert2;
	PyObject *op1, *op2;
	int n1, n2;	
	if (PyArg_ParseTuple(args, "dddd", &arg1, &arg2, &arg3, &arg4)) 	
  		glRectd(arg1, arg2, arg3, arg4);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "OO", &op1, &op2));
		TRY (PyArray_AsDoubleArray(&op1, &vert1, &n1));
		TRY (PyArray_AsDoubleArray(&op2, &vert2, &n2));
		if ((n1 < 2) || (n2 < 2)) {
			PyErr_SetString(gl_Error,  "need elements with at least 2 items");
			Py_DECREF(op1);
			Py_DECREF(op2);
			return NULL;
		}
		glRectdv(vert1, vert2);
		Py_DECREF(op1);
		Py_DECREF(op2);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Rectf(PyObject* self, PyObject* args)
{
	GLfloat arg1, arg2, arg3, arg4, *vert1, *vert2;
	PyObject *op1, *op2;
	int n1, n2;	
	if (PyArg_ParseTuple(args, "ffff", &arg1, &arg2, &arg3, &arg4)) 	
  		glRectf(arg1, arg2, arg3, arg4);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "OO", &op1, &op2));
		TRY (PyArray_AsFloatArray(&op1, &vert1, &n1));
		TRY (PyArray_AsFloatArray(&op2, &vert2, &n2));
		if ((n1 < 2) || (n2 < 2)) {
			PyErr_SetString(gl_Error,  "need elements with at least 2 items");
			Py_DECREF(op1);
			Py_DECREF(op2);
			return NULL;
		}
		glRectfv(vert1, vert2);
		Py_DECREF(op1);
		Py_DECREF(op2);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Recti(PyObject* self, PyObject* args)
{
	GLint arg1, arg2, arg3, arg4, *vert1, *vert2;
	PyObject *op1, *op2;
	int n1, n2;	
	if (PyArg_ParseTuple(args, "iiii", &arg1, &arg2, &arg3, &arg4)) 	
  		glRecti(arg1, arg2, arg3, arg4);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "OO", &op1, &op2));
		TRY (PyArray_AsIntArray(&op1, &vert1, &n1));
		TRY (PyArray_AsIntArray(&op2, &vert2, &n2));
		if ((n1 < 2) || (n2 < 2)) {
			PyErr_SetString(gl_Error,  "need elements with at least 2 items");
			Py_DECREF(op1);
			Py_DECREF(op2);
			return NULL;
		}
		glRectiv(vert1, vert2);
		Py_DECREF(op1);
		Py_DECREF(op2);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Rects(PyObject* self, PyObject* args)
{
	GLshort arg1, arg2, arg3, arg4, *vert1, *vert2;
	PyObject *op1, *op2;
	int n1, n2;	
	if (PyArg_ParseTuple(args, "hhhh", &arg1, &arg2, &arg3, &arg4)) 	
  		glRects(arg1, arg2, arg3, arg4);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "OO", &op1, &op2));
		TRY (PyArray_AsShortArray(&op1, &vert1, &n1));
		TRY (PyArray_AsShortArray(&op2, &vert2, &n2));
		if ((n1 < 2) || (n2 < 2)) {
			PyErr_SetString(gl_Error,  "need elements with at least 2 items");
			Py_DECREF(op1);
			Py_DECREF(op2);
			return NULL;
		}
		glRectsv(vert1, vert2);
		Py_DECREF(op1);
		Py_DECREF(op2);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Rotated(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, arg3, arg4;
	TRY (PyArg_ParseTuple(args, "dddd", &arg1, &arg2, &arg3, &arg4));
  	glRotated(arg1, arg2, arg3, arg4);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Rotatef(PyObject* self, PyObject* args)
{
	GLfloat arg1, arg2, arg3, arg4;
	TRY (PyArg_ParseTuple(args, "ffff", &arg1, &arg2, &arg3, &arg4));
  	glRotatef(arg1, arg2, arg3, arg4);
	Py_INCREF(Py_None);
	return Py_None;
}


static PyObject* gl_Scaled(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, arg3;
	TRY (PyArg_ParseTuple(args, "ddd", &arg1, &arg2, &arg3));
  	glScaled(arg1, arg2, arg3);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Scalef(PyObject* self, PyObject* args)
{
	GLfloat arg1, arg2, arg3;
	TRY (PyArg_ParseTuple(args, "fff", &arg1, &arg2, &arg3));
  	glScalef(arg1, arg2, arg3);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Scissor(PyObject* self, PyObject* args)
{
	GLint arg1, arg2;
	GLsizei arg3, arg4;
	TRY (PyArg_ParseTuple(args, "iiii", &arg1, &arg2, &arg3, &arg4));
  	glScissor(arg1, arg2, arg3, arg4);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_ShadeModel(PyObject* self, PyObject* args)
{
	GLenum arg1;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glShadeModel(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_StencilFunc(PyObject* self, PyObject* args)
{
	GLenum arg1;
	GLint arg2;
	GLuint arg3;
	TRY (PyArg_ParseTuple(args, "iii", &arg1, &arg2, &arg3));
  	glStencilFunc(arg1, arg2, arg3);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_StencilMask(PyObject* self, PyObject* args)
{
	GLuint arg1;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glStencilMask(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_StencilOp(PyObject* self, PyObject* args)
{
	GLenum arg1, arg2, arg3;
	TRY (PyArg_ParseTuple(args, "iii", &arg1, &arg2, &arg3));
  	glStencilOp(arg1, arg2, arg3);
	Py_INCREF(Py_None);
	return Py_None;
}


static PyObject* gl_TexCoord1d(PyObject* self, PyObject* args)
{
	GLdouble arg1;
	TRY (PyArg_ParseTuple(args, "d", &arg1));
  	glTexCoord1d(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexCoord1f(PyObject* self, PyObject* args)
{
	GLfloat arg1;
	TRY (PyArg_ParseTuple(args, "f", &arg1));
  	glTexCoord1f(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexCoord1i(PyObject* self, PyObject* args)
{
	GLint arg1;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glTexCoord1i(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexCoord1s(PyObject* self, PyObject* args)
{
	GLshort arg1;
	TRY (PyArg_ParseTuple(args, "h", &arg1));
  	glTexCoord1s(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexCoord2d(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "dd", &arg1, &arg2)) 	
  		glTexCoord2d(arg1, arg2);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsDoubleArray(&op, &vert, &n));
		if (n < 2) {
			PyErr_SetString(gl_Error,  "need element with at least 2 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glTexCoord2dv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexCoord2f(PyObject* self, PyObject* args)
{
	GLfloat arg1, arg2, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "ff", &arg1, &arg2)) 	
  		glTexCoord2f(arg1, arg2);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsFloatArray(&op, &vert, &n));
		if (n < 2) {
			PyErr_SetString(gl_Error,  "need element with at least 2 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glTexCoord2fv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexCoord2i(PyObject* self, PyObject* args)
{
	GLint arg1, arg2, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "ii", &arg1, &arg2)) 	
  		glTexCoord2i(arg1, arg2);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsIntArray(&op, &vert, &n));
		if (n < 2) {
			PyErr_SetString(gl_Error,  "need element with at least 2 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glTexCoord2iv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexCoord2s(PyObject* self, PyObject* args)
{
	GLshort arg1, arg2, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "hh", &arg1, &arg2)) 	
  		glTexCoord2s(arg1, arg2);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsShortArray(&op, &vert, &n));
		if (n < 2) {
			PyErr_SetString(gl_Error,  "need element with at least 2 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glTexCoord2sv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexCoord3d(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, arg3, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "ddd", &arg1, &arg2, &arg3)) 	
  		glTexCoord3d(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsDoubleArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glTexCoord3dv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexCoord3f(PyObject* self, PyObject* args)
{
	GLfloat arg1, arg2, arg3, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "fff", &arg1, &arg2, &arg3)) 	
  		glTexCoord3f(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsFloatArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glTexCoord3fv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexCoord3i(PyObject* self, PyObject* args)
{
	GLint arg1, arg2, arg3, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "iii", &arg1, &arg2, &arg3)) 	
  		glTexCoord3i(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsIntArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glTexCoord3iv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexCoord3s(PyObject* self, PyObject* args)
{
	GLshort arg1, arg2, arg3, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "hhh", &arg1, &arg2, &arg3)) 	
  		glTexCoord3s(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsShortArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glTexCoord3sv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexCoord4d(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, arg3, arg4, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "dddd", &arg1, &arg2, &arg3, &arg4)) 	
  		glTexCoord4d(arg1, arg2, arg3, arg4);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsDoubleArray(&op, &vert, &n));
		if (n < 4) {
			PyErr_SetString(gl_Error,  "need element with at least 4 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glTexCoord4dv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexCoord4f(PyObject* self, PyObject* args)
{
	GLfloat arg1, arg2, arg3, arg4, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "ffff", &arg1, &arg2, &arg3, &arg4)) 	
  		glTexCoord4f(arg1, arg2, arg3, arg4);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsFloatArray(&op, &vert, &n));
		if (n < 4) {
			PyErr_SetString(gl_Error,  "need element with at least 4 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glTexCoord4fv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexCoord4i(PyObject* self, PyObject* args)
{
	GLint arg1, arg2, arg3, arg4, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "iiii", &arg1, &arg2, &arg3, &arg4)) 	
  		glTexCoord4i(arg1, arg2, arg3, arg4);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsIntArray(&op, &vert, &n));
		if (n < 4) {
			PyErr_SetString(gl_Error,  "need element with at least 4 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glTexCoord4iv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexCoord4s(PyObject* self, PyObject* args)
{
	GLshort arg1, arg2, arg3, arg4, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "hh", &arg1, &arg2, &arg3, &arg4)) 	
  		glTexCoord4s(arg1, arg2, arg3, arg4);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsShortArray(&op, &vert, &n));
		if (n < 4) {
			PyErr_SetString(gl_Error,  "need element with at least 4 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glTexCoord4sv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexEnvf(PyObject* self, PyObject* args)
{
	GLenum arg1, arg2;
	GLfloat arg3, *rest;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "iif", &arg1, &arg2, &arg3))
  		glTexEnvf(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "iiO", &arg1, &arg2, &op));
		TRY (PyArray_AsFloatArray(&op, &rest, &n));
		if ((n == 1) || (n > 3)) {
			glTexEnvfv(arg1, arg2, rest);
			PyArray_ClearMemory(op, rest);
		}
		else {
			PyErr_SetString(gl_Error,  "3. argument needs 1 or at least 4 items!");
			PyArray_ClearMemory(op, rest);
			return NULL;
		}
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexEnvi(PyObject* self, PyObject* args)
{
	GLenum arg1, arg2;
	GLint arg3, *rest;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "iii", &arg1, &arg2, &arg3))
  		glTexEnvi(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "iiO", &arg1, &arg2, &op));
		TRY (PyArray_AsIntArray(&op, &rest, &n));
		if ((n == 1) || (n > 3)) {
			glTexEnviv(arg1, arg2, rest);
			PyArray_ClearMemory(op, rest);
		}
		else {
			PyErr_SetString(gl_Error,  "3. argument needs 1 or at least 4 items!");
			PyArray_ClearMemory(op, rest);
			return NULL;
		}
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexGend(PyObject* self, PyObject* args)
{
	GLenum arg1, arg2;
	GLdouble arg3, *rest;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "iid", &arg1, &arg2, &arg3))
  		glTexGend(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "iiO", &arg1, &arg2, &op));
		TRY (PyArray_AsDoubleArray(&op, &rest, &n));
		if ((n == 1) || (n>3)) {
			glTexGendv(arg1, arg2, rest);
			PyArray_ClearMemory(op, rest);
		}
		else {
			PyErr_SetString(gl_Error,  "3. argument needs 1 or at least 4 items!");
			PyArray_ClearMemory(op, rest);
			return NULL;
		}
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexGenf(PyObject* self, PyObject* args)
{
	GLenum arg1, arg2;
	GLfloat arg3, *rest;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "iif", &arg1, &arg2, &arg3))
  		glTexGenf(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "iiO", &arg1, &arg2, &op));
		TRY (PyArray_AsFloatArray(&op, &rest, &n));
		if ((n == 1) || (n > 3)) {
			glTexGenfv(arg1, arg2, rest);
			PyArray_ClearMemory(op, rest);
		}
		else {
			PyErr_SetString(gl_Error,  "3. argument needs 1 or at least 4 items!");
			PyArray_ClearMemory(op, rest);
			return NULL;
		}
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexGeni(PyObject* self, PyObject* args)
{
	GLenum arg1, arg2;
	GLint arg3, *rest;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "iii", &arg1, &arg2, &arg3))
  		glTexGeni(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "iiO", &arg1, &arg2, &op));
		TRY (PyArray_AsIntArray(&op, &rest, &n));
		if ((n == 1) || (n>3)) {
			glTexGeniv(arg1, arg2, rest);
			PyArray_ClearMemory(op, rest);
		}
		else {
			PyErr_SetString(gl_Error,  "3. argument needs 1 or at least 4 items!");
			PyArray_ClearMemory(op, rest);
			return NULL;
		}
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexParameterf(PyObject* self, PyObject* args)
{
	GLenum arg1, arg2;
	GLfloat arg3, *rest;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "iif", &arg1, &arg2, &arg3))
  		glTexParameterf(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "iiO", &arg1, &arg2, &op));
		TRY (PyArray_AsFloatArray(&op, &rest, &n));
		if ((n == 1) || (n > 3)) {
			glTexParameterfv(arg1, arg2, rest);
			PyArray_ClearMemory(op, rest);
		}
		else {
			PyErr_SetString(gl_Error,  "3. argument needs 1 or at least 4 items!");
			PyArray_ClearMemory(op, rest);
			return NULL;
		}
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_TexParameteri(PyObject* self, PyObject* args)
{
	GLenum arg1, arg2;
	GLint arg3, *rest;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "iii", &arg1, &arg2, &arg3))
  		glTexParameteri(arg1, arg2, arg3);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "iiO", &arg1, &arg2, &op));
		TRY (PyArray_AsIntArray(&op, &rest, &n));
		if ((n == 1) || (n > 3)) {
			glTexParameteriv(arg1, arg2, rest);
			PyArray_ClearMemory(op, rest);
		}
		else {
			PyErr_SetString(gl_Error,  "3. argument needs 1 or at least 4 items!");
			PyArray_ClearMemory(op, rest);
			return NULL;
		}
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Translated(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, arg3;
	TRY (PyArg_ParseTuple(args, "ddd", &arg1, &arg2, &arg3));
  	glTranslated(arg1, arg2, arg3);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Translatef(PyObject* self, PyObject* args)
{
	GLfloat arg1, arg2, arg3;
	TRY (PyArg_ParseTuple(args, "fff", &arg1, &arg2, &arg3));
  	glTranslatef(arg1, arg2, arg3);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Vertex2d(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, *vert;
	PyObject *op;
	int n;	
	if (PyArg_ParseTuple(args, "dd", &arg1, &arg2)) 	
  		glVertex2d(arg1, arg2);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsDoubleArray(&op, &vert, &n));
		if (n < 2) {
			PyErr_SetString(gl_Error,  "need element with at least 2 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glVertex2dv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

/* experimental */

static PyObject* gl_ColorVertex2d(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, *vert;
	PyObject *op1, *colors=NULL;
	int n;	
#ifdef NUMERIC
	PyObject *op2;
	PyArrayObject *cp;
	int ok1, ok2, nc, m, i;
	GLdouble *x, *y, *c;
#endif
	if (PyArg_ParseTuple(args, "dd", &arg1, &arg2)) 	
  		glVertex2d(arg1, arg2);
	else {
		PyErr_Clear();
		if (PyArg_ParseTuple(args, "O", &op1)) {
			TRY (PyArray_AsDoubleArray(&op1, &vert, &n));
			if (n < 2) {
				PyErr_SetString(gl_Error, "need element with at least 2 items");
				PyArray_ClearMemory(op1, vert);
				return NULL;
			}
			glVertex2dv(vert);
			PyArray_ClearMemory(op1, vert);
		}
#ifdef NUMERIC
		else {
			PyErr_Clear();
			TRY (PyArg_ParseTuple(args, "OO|O", &op1, &op2, &colors)); 
			ok1 = PyArray_AsDoubleArray(&op1, &x, &m);
			ok2 = PyArray_AsDoubleArray(&op2, &y, &n);
			if ((m != n) || (!ok1) || (!ok2)) {
					PyErr_SetString(gl_Error, "coordinate arrays must be of same length or not enough memory");
					PyArray_ClearMemory(op1, x);
					PyArray_ClearMemory(op2, y);
					return NULL;
			}
			if (colors) {
				if (!(cp =(PyArrayObject *)PyArray_ContiguousFromObject(colors, 
																		PyArray_DOUBLE, 1, 2))) {
					PyArray_ClearMemory(op1, x);
					PyArray_ClearMemory(op2, y);
					return NULL;
				}
   			c = (GLdouble *)cp->data;
				nc = PyArray_Size((PyObject *)cp);
				if ( ((nc%3) != 0) || (n != nc/3)) {
					PyErr_SetString(gl_Error, "wrong color matrix size");
					PyArray_ClearMemory(op1, x);
					PyArray_ClearMemory(op2, y);
					PyArray_ClearMemory(cp, c);
					return NULL;
				}

				for (i=0; i<n; i++) {
					glColor3dv(c); c += 3;
					glVertex2d(*x++, *y++);
				}
			}
			else {
				for (i=0; i<n; i++) {
					glVertex2d(*x++, *y++);
				}
			}
			PyArray_ClearMemory(op1, x);
			PyArray_ClearMemory(op2, y);	
			PyArray_ClearMemory(cp, c);
		}
	}
#else
	}
#endif
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Vertex2f(PyObject* self, PyObject* args)
{
	GLfloat arg1, arg2;
#ifdef NUMERIC
	GLfloat *vert;
	PyObject *op;
	int n;	
#endif
	if (PyArg_ParseTuple(args, "ff", &arg1, &arg2)) 	
  		glVertex2f(arg1, arg2);
#ifdef NUMERIC
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsFloatArray(&op, &vert, &n));
		if (n < 2) {
			PyErr_SetString(gl_Error,  "need element with at least 2 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glVertex2fv(vert);
		PyArray_ClearMemory(op, vert);
	}
#endif
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Vertex2i(PyObject* self, PyObject* args)
{
	GLint arg1, arg2;
#ifdef NUMERIC
	GLint *vert;
	PyObject *op;
	int n;	
#endif
	if (PyArg_ParseTuple(args, "ii", &arg1, &arg2)) 	
  		glVertex2i(arg1, arg2);
#ifdef NUMERIC
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsIntArray(&op, &vert, &n));
		if (n < 2) {
			PyErr_SetString(gl_Error,  "need element with at least 2 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glVertex2iv(vert);
		PyArray_ClearMemory(op, vert);
	}
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Vertex2s(PyObject* self, PyObject* args)
{
	GLshort arg1, arg2;
#ifdef NUMERIC
	GLshort *vert;
	PyObject *op;
	int n;	
#endif
	if (PyArg_ParseTuple(args, "hh", &arg1, &arg2)) 	
  		glVertex2s(arg1, arg2);
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsShortArray(&op, &vert, &n));
		if (n < 2) {
			PyErr_SetString(gl_Error,  "need element with at least 2 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glVertex2sv(vert);
		PyArray_ClearMemory(op, vert);
	}
#endif
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Vertex3d(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, arg3;
#ifdef NUMERIC
	GLdouble *vert;
	PyObject *op;
	int n;	
#endif
	if (PyArg_ParseTuple(args, "ddd", &arg1, &arg2, &arg3)) 	
  		glVertex3d(arg1, arg2, arg3);
#ifdef NUMERIC
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsDoubleArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glVertex3dv(vert);
		PyArray_ClearMemory(op, vert);
	}
#endif
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Vertex3f(PyObject* self, PyObject* args)
{
	GLfloat arg1, arg2, arg3;
#ifdef NUMERIC
	GLfloat *vert;
	PyObject *op;
	int n;	
#endif
	if (PyArg_ParseTuple(args, "fff", &arg1, &arg2, &arg3)) 	
  		glVertex3f(arg1, arg2, arg3);
#ifdef NUMERIC
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsFloatArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glVertex3fv(vert);
		PyArray_ClearMemory(op, vert);
	}
#endif
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Vertex3i(PyObject* self, PyObject* args)
{
	GLint arg1, arg2, arg3;
#ifdef NUMERIC
	GLint *vert;
	PyObject *op;
	int n;	
#endif
	if (PyArg_ParseTuple(args, "iii", &arg1, &arg2, &arg3)) 	
  		glVertex3i(arg1, arg2, arg3);
#ifdef NUMERIC
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsIntArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glVertex3iv(vert);
		PyArray_ClearMemory(op, vert);
	}
#endif
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Vertex3s(PyObject* self, PyObject* args)
{
	GLshort arg1, arg2, arg3;
#ifdef NUMERIC
	GLshort *vert;
	PyObject *op;
	int n;	
#endif
	if (PyArg_ParseTuple(args, "hhh", &arg1, &arg2, &arg3)) 	
  		glVertex3s(arg1, arg2, arg3);
#ifdef NUMERIC
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsShortArray(&op, &vert, &n));
		if (n < 3) {
			PyErr_SetString(gl_Error,  "need element with at least 3 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glVertex3sv(vert);
		PyArray_ClearMemory(op, vert);
	}
#endif
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Vertex4d(PyObject* self, PyObject* args)
{
	GLdouble arg1, arg2, arg3, arg4;
#ifdef NUMERIC
	GLdouble *vert;
	PyObject *op;
	int n;	
#endif
	if (PyArg_ParseTuple(args, "dddd", &arg1, &arg2, &arg3, &arg4)) 	
  		glVertex4d(arg1, arg2, arg3, arg4);
#ifdef NUMERIC
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsDoubleArray(&op, &vert, &n));
		if (n < 4) {
			PyErr_SetString(gl_Error,  "need element with at least 4 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glVertex4dv(vert);
		PyArray_ClearMemory(op, vert);
	}
#endif
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Vertex4f(PyObject* self, PyObject* args)
{
	GLfloat arg1, arg2, arg3, arg4;
#ifdef NUMERIC
	GLfloat *vert;
	PyObject *op;
	int n;	
#endif
	if (PyArg_ParseTuple(args, "ffff", &arg1, &arg2, &arg3, &arg4)) 	
  		glVertex4f(arg1, arg2, arg3, arg4);
#ifdef NUMERIC
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsFloatArray(&op, &vert, &n));
		if (n < 4) {
			PyErr_SetString(gl_Error,  "need element with at least 4 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glVertex4fv(vert);
		PyArray_ClearMemory(op, vert);
	}
#endif
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Vertex4i(PyObject* self, PyObject* args)
{
	GLint arg1, arg2, arg3, arg4;
#ifdef NUMERIC
	GLint *vert;
	PyObject *op;
	int n;	
#endif
	if (PyArg_ParseTuple(args, "iiii", &arg1, &arg2, &arg3, &arg4)) 	
  		glVertex4i(arg1, arg2, arg3, arg4);
#ifdef NUMERIC
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsIntArray(&op, &vert, &n));
		if (n < 4) {
			PyErr_SetString(gl_Error,  "need element with at least 4 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glVertex4iv(vert);
		PyArray_ClearMemory(op, vert);
	}
#endif
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Vertex4s(PyObject* self, PyObject* args)
{
	GLshort arg1, arg2, arg3, arg4;
#ifdef NUMERIC
	GLshort *vert;
	PyObject *op;
	int n;	
#endif
	if (PyArg_ParseTuple(args, "hhhh", &arg1, &arg2, &arg3, &arg4)) 	
  		glVertex4s(arg1, arg2, arg3, arg4);
#ifdef NUMERIC
	else {
		PyErr_Clear();
		TRY (PyArg_ParseTuple(args, "O", &op));
		TRY (PyArray_AsShortArray(&op, &vert, &n));
		if (n < 4) {
			PyErr_SetString(gl_Error,  "need element with at least 4 items");
			PyArray_ClearMemory(op, vert);
			return NULL;
		}
		glVertex4sv(vert);
		PyArray_ClearMemory(op, vert);
	}
#endif
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_Viewport(PyObject* self, PyObject* args)
{
	GLint arg1, arg2;
	GLsizei arg3, arg4;
	TRY (PyArg_ParseTuple(args, "iiii", &arg1, &arg2, &arg3, &arg4));
  	glViewport(arg1 , arg2 , arg3 , arg4);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_ClipPlane(PyObject* self, PyObject* args)
{
  	GLenum arg1;
  	PyObject *eop;
	GLdouble *equation;
	int n;
	TRY (PyArg_ParseTuple(args, "iO", &arg1, &eop));
	TRY (PyArray_AsDoubleArray(&eop, &equation, &n));
	if (n < 4) {
		PyErr_SetString(gl_Error,  "second argument needs at least 4 items");
		Py_DECREF(eop);
		return NULL;
		}
  	glClipPlane(arg1, equation);
	Py_DECREF(eop);      
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_GetClipPlane(PyObject* self, PyObject* args)
{
  	GLenum arg1;
#ifdef NUMERIC
  	PyArrayObject *values;
#else /* Not NUMERIC */
	PyObject *values;
	int i;
#endif /* Not NUMERIC */
	GLdouble equation[4];
	int dims[1];
	TRY (PyArg_ParseTuple(args, "i", &arg1));
	glGetClipPlane(arg1, equation);
	dims[0] = 4;
#ifdef NUMERIC
	values = (PyArrayObject *) PyArray_FromDims (1, dims, PyArray_DOUBLE);
	memcpy(values->data, equation, 4*sizeof(double));
#else /* Not NUMERIC */
	values = PyTuple_New(4);
	for (i = 0; i < 4; i++) {
	  PyTuple_SET_ITEM(values, i, PyFloat_FromDouble(equation[i]));
	}
#endif /* Not NUMERIC */
	return (PyObject*) values;
}


static PyObject* gl_IsEnabled(PyObject* self, PyObject* args)
{
  	GLenum arg1;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	return PyInt_FromLong(glIsEnabled(arg1));
}

static PyObject* gl_IsList(PyObject* self, PyObject* args)
{
  	GLuint arg1;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	return PyInt_FromLong(glIsList(arg1));
}

static PyObject* gl_RenderMode(PyObject* self, PyObject* args)
{
  	GLenum arg1;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	return PyInt_FromLong(glRenderMode(arg1));
}

static PyObject* gl_GenLists(PyObject* self, PyObject* args)
{
  	GLsizei arg1;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	return PyInt_FromLong(glGenLists(arg1));
}

static PyObject* gl_GetDoublev(PyObject* self, PyObject* args)
{
  GLenum pname;
  GLdouble oneparam;
  int nd=1, dims[3];
#ifdef NUMERIC
  PyArrayObject* params;
#else /* NUMERIC */
  int i, nitems;
  GLdouble *items;
  PyObject *params;
#endif /* NUMERIC */
  if (!PyArg_ParseTuple(args, "i", &pname))
    return NULL;
  switch(pname) {
    case GL_ACCUM_CLEAR_VALUE:
#ifdef GL_BLEND_COLOR_EXT
    case GL_BLEND_COLOR_EXT:
#endif
    case GL_COLOR_CLEAR_VALUE:
    case GL_COLOR_WRITEMASK:
    case GL_CURRENT_COLOR:
    case GL_CURRENT_RASTER_COLOR:
    case GL_CURRENT_RASTER_POSITION:
    case GL_CURRENT_RASTER_TEXTURE_COORDS:
    case GL_CURRENT_TEXTURE_COORDS:
    case GL_FOG_COLOR:
    case GL_LIGHT_MODEL_AMBIENT:
    case GL_MAP2_GRID_DOMAIN:
    case GL_SCISSOR_BOX:
    case GL_TEXTURE_ENV_COLOR:
    case GL_VIEWPORT:
	dims[0] = 4;
	break;
    case GL_CURRENT_NORMAL:
	dims[0] = 3;
	break;
    case GL_DEPTH_RANGE:	
    case GL_LINE_WIDTH_RANGE:
    case GL_MAP1_GRID_DOMAIN:
    case GL_MAP2_GRID_SEGMENTS:
    case GL_MAX_VIEWPORT_DIMS:
    case GL_POINT_SIZE_RANGE:
    case GL_POLYGON_MODE:
	dims[0] = 2;
	break;
    case GL_MODELVIEW_MATRIX:
    case GL_PROJECTION_MATRIX:
    case GL_TEXTURE_MATRIX:
	nd = 2;
	dims[0] = 4; dims[1] = 4;
	break;
    case GL_POLYGON_STIPPLE:
	dims[0] = 32;
	break;
    default:
        glGetDoublev(pname, &oneparam);
	return Py_BuildValue("d", oneparam);  
    }
#ifdef NUMERIC
    TRY(params = (PyArrayObject *)PyArray_FromDims(nd, dims, PyArray_DOUBLE));
    glGetDoublev(pname, (double *)params->data);
#else /* NUMERIC */
    nitems = 1;
    for(i = 0; i < nd; i++) {
      nitems *= dims[i];
    }
    TRY(items = PyMem_NEW(GLdouble, nitems));
    glGetDoublev(pname, items);
    TRY(params = PyTuple_New(nitems));
    for (i = 0; i < nitems; i++) {
      PyTuple_SET_ITEM(params, i, PyFloat_FromDouble(items[i]));
    }
    PyMem_DEL(items);
#endif /* NUMERIC */
    return (PyObject *)params;
}

/*OpenGL Extensions*/

static PyObject* gl_PolygonOffsetEXT(PyObject* self, PyObject* args)
{
#ifdef GL_EXT_polygon_offset
	GLfloat arg1, arg2;
	TRY (PyArg_ParseTuple(args, "ff", &arg1, &arg2));
  	glPolygonOffsetEXT(arg1, arg2);
 	Py_INCREF(Py_None);
	return Py_None;
#else
	PyErr_SetString(PyExc_ValueError, "PolygonOffsetEXT not implemented");
	return NULL;
#endif
}

static PyObject* gl_ArrayElementEXT(PyObject* self, PyObject* args)
{
#ifdef GL_EXT_array_element
  	GLint arg1;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glArrayElementEXT(arg1);
	Py_INCREF(Py_None);
	return Py_None;
#else
	PyErr_SetString(PyExc_ValueError, "ArrayElementEXT not implemented");
	return NULL;
#endif
}

/*These need to be fixed to match the above*/
#ifdef NOT_DEFINED
static PyObject* gl_DrawArraysEXT(PyObject* self, PyObject* args)
{
  	GLenum arg1;
  	GLint arg2;
	GLsizei arg3;
	TRY (PyArg_ParseTuple(args, "iii", &arg1, &arg2, &arg3));
  	glDrawArraysEXT(arg1, arg2, arg3);
	Py_INCREF(Py_None);
	return Py_None;
}
static PyObject* gl_BlendColorEXT(PyObject* self, PyObject* args)
{
  	GLclampf arg1, arg2, arg3, arg4;
	TRY (PyArg_ParseTuple(args, "ffff", &arg1, &arg2, &arg3, &arg4));
  	glBlendColorEXT(arg1, arg2, arg3, arg4);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_BlendEquationEXT(PyObject* self, PyObject* args)
{
  	GLenum arg1;
	TRY (PyArg_ParseTuple(args, "i", &arg1));
  	glBlendEquationEXT(arg1);
	Py_INCREF(Py_None);
	return Py_None;
}
#endif

/*Need to find proper size for glReadPixels array*/
static int glformat_size(GLenum format) {
	switch(format) {
	case GL_COLOR_INDEX:
	case GL_RED:
	case GL_GREEN:
	case GL_BLUE:
	case GL_ALPHA:
	case GL_STENCIL_INDEX:
	case GL_DEPTH_COMPONENT:
	case GL_LUMINANCE:
		return 1;

	case GL_LUMINANCE_ALPHA:
		return 2;

	case GL_RGB:
#ifdef GL_BGR_EXT
	case GL_BGR_EXT:
#endif
		return 3;

	case GL_RGBA:
#ifdef GL_BGRA_EXT
	case GL_BGRA_EXT:
#endif
#ifdef GL_ABGR_EXT
	case GL_ABGR_EXT:
#endif
		return 4;
	default:
		return -1;
	}
}

static int gltype_size(GLenum type) {
	switch(type) {
	case GL_BYTE:
	case GL_UNSIGNED_BYTE:
		return 8;

	case GL_SHORT:
	case GL_UNSIGNED_SHORT:
		return 16;

	case GL_INT:
	case GL_UNSIGNED_INT:
	case GL_FLOAT:
		return 32;

	case GL_BITMAP:
		return 1;
	
	default:
		return -1;
	}
}

static PyObject* gl_DrawPixels(PyObject* self, PyObject* args)
{
    char *data;  
    int format, type, width, height, n;
	int format_size, type_size, total_size;
	
    TRY (PyArg_ParseTuple(args, "iiiis#", &width, &height, &format, &type,
			&data, &n));
	/*This is a poor substitute for the RIGHT solution*/
	format_size = glformat_size(format);
	ASSERT(format_size != -1, "invalid format");
	type_size = gltype_size(type);
	ASSERT(format_size != -1, "invalid type");
	total_size = type_size*format_size*width*height/8;
	ASSERT(total_size >= n, "data area too small");

	glDrawPixels(width, height, format, type, data);

    Py_INCREF(Py_None);
    return Py_None; 
}


#ifdef NUMERIC
static PyObject* gl_TexImage2D(PyObject* self, PyObject* args)
{
  GLenum arg1;
  GLint arg2;
  GLint arg3;
  GLsizei arg4;
  GLsizei arg5;
  GLint arg6;
  GLenum arg7;
  GLenum arg8;
  PyObject *vop;
  PyArrayObject *ap;  
  char *data; 
  int format, type;
  int format_size, type_size, total_size;
  
  TRY (PyArg_ParseTuple(args, "iiiiiiiiO", &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7, &arg8, &vop));

  if (PyString_Check(vop)) {
	data = PyString_AsString(vop);
	format_size = glformat_size(arg7);
	ASSERT(format_size != -1, "invalid format");
	type_size = gltype_size(arg8);
	ASSERT(format_size != -1, "invalid type");
	total_size = type_size*format_size*arg4*arg5/8;
	ASSERT(total_size >= PyString_Size(vop), "data area too small");

	glTexImage2D(arg1, arg2, arg3, arg4, arg5, arg6,
					arg7, arg8, data);
  } else {
	  if (PyArray_Check(vop)) {
		ap = (PyArrayObject *)vop;
		ASSERT(ap->nd == 2 || ap->nd == 3, "array must be either 2 or 3d");
	  } else {
		TRY(ap = (PyArrayObject *)PyArray_ContiguousFromObject(vop, PyArray_UBYTE, 2, 3));
	  }
  
	  type = typecode2gltype[ap->descr->type_num];
	  ASSERT(type != -1, "can't convert this type of array to an image");
  
	  if (ap->nd == 2) {
		format = GL_LUMINANCE;
	  } else {
		ASSERT(ap->dimensions[2] == 3 || ap->dimensions[2] == 4, "3d array must be RGB or RGBA");
    
		if (ap->dimensions[2] == 3) {
		  format = GL_RGB;
		} else {
		  format = GL_RGBA;
		  }
	  }
	  glTexImage2D(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, ap->data);
  }
  Py_INCREF(Py_None);
  return Py_None;
}
#endif

static PyObject* gl_ReadPixels(PyObject* self, PyObject* args)
{
	PyObject *sp;
    char *data;  
	int x, y;
    int format, type, width, height, n=-1;
	int format_size, type_size, total_size;
	
    TRY (PyArg_ParseTuple(args, "iiiiii|i", &x, &y, &width, &height, &format, &type, &n));
	/*This is a poor substitute for the RIGHT solution*/
	format_size = glformat_size(format);
	ASSERT(format_size != -1, "invalid format");
	type_size = gltype_size(type);
	ASSERT(format_size != -1, "invalid type");
	total_size = type_size*format_size*width*height/8;
	if (n == -1) n = total_size;
	ASSERT(total_size >= n, "data area too small");

	data = (char *)malloc(n);
	glReadPixels(x, y, width, height, format, type, data);
	sp = PyString_FromStringAndSize(data, n);
	free(data);
	return sp;
}

#ifdef GL_VERSION_1_1
#define OPENGL1_1
#endif

/* 
############################### OpenGL 1.1 ########################
*/
#ifdef OPENGL1_1
#ifdef NUMERIC

static int type2gl[] = {-1, GL_UNSIGNED_BYTE, GL_BYTE, GL_SHORT, GL_INT, -1, GL_FLOAT, GL_DOUBLE, -1, -1, -1, -1};

static PyObject* gl_VertexPointer(PyObject* self, PyObject* args)
{
	GLint size;
	GLenum type;
	GLsizei stride;
	PyObject *op;
	PyArrayObject *ap;	
	TRY (PyArg_ParseTuple(args, "iiO", &size, &stride, &op)); 
	if (PyArray_Check(op)) 
		ap = (PyArrayObject *)op;
	else {
		TRY(ap = (PyArrayObject *)PyArray_ContiguousFromObject(op, PyArray_DOUBLE, 1, 0));
	}
	type = type2gl[ap->descr->type_num];
	ASSERT(type != -1, "Can't convert this type of array!");
	glVertexPointer(size, type, stride, ap->data);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_ColorPointer(PyObject* self, PyObject* args)
{
	GLint size;
	GLenum type;
	GLsizei stride;
	PyObject *op;
	PyArrayObject *ap;	
	TRY (PyArg_ParseTuple(args, "iiO", &size, &stride, &op)); 
	if (PyArray_Check(op)) 
		ap = (PyArrayObject *)op;
	else {
		TRY(ap = (PyArrayObject *)PyArray_ContiguousFromObject(op, PyArray_DOUBLE, 1, 0));
	}
	type = type2gl[ap->descr->type_num];
	ASSERT(type != -1, "Can't convert this type of array!");
	glColorPointer(size, type, stride, ap->data);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject* gl_DrawArrays(PyObject* self, PyObject* args)
{
	GLenum mode;
	GLint first;
	GLsizei count;
	TRY (PyArg_ParseTuple(args, "iii", &mode, &first, &count)); 
	glDrawArrays(mode, first, count);
	Py_INCREF(Py_None);
	return Py_None;
}

#endif /* NUMERIC */
#endif /* OPENGL1_1 */

/* Michel Sanner Jan 14 1998 */

/*
    Check for GL_ERRORS and returns error code or None
*/
static PyObject *gl_GetError(PyObject *self, PyObject *args)
{
  GLenum errCode = glGetError();

  if (errCode == GL_NO_ERROR){
    Py_INCREF(Py_None);
    return Py_None;
  } else {
    return Py_BuildValue("i", errCode);
  }
}

#ifdef NUMERIC
/*
   Allocate a numeric array of a user specified size to hold hardware picking
   event hits. This "Numeric" array is return to the interpreter.
   OpenGL 1.0, requires Numeric extension.

   available from the interpreter as:
      SelectBuffer( int size )
*/
static PyObject* gl_SelectBuffer(PyObject* self, PyObject* args)
{
    int size;
    PyArrayObject *buf;

    TRY (PyArg_ParseTuple(args, "i", &size));   	     

    TRY(buf = (PyArrayObject *)PyArray_FromDims(1, &size, PyArray_INT));

    glSelectBuffer (buf->dimensions[0], (GLuint *)buf->data);    

    return (PyObject *)buf;
}
#endif


/* END Michel Sanner */



/* 
########################################################################
*/

static PyMethodDef gl_methods[] = {
  	{ "glAccum",		gl_Accum,			1},
  	{ "glAlphaFunc",		gl_AlphaFunc, 			1},
  	{ "glBegin",		gl_Begin, 			1},
  	{ "glBlendFunc",		gl_BlendFunc, 			1},
  	{ "glCallList",       	gl_CallList, 			1},
  	{ "glClear",          	gl_Clear,			1},
  	{ "glClearAccum",     	gl_ClearAccum, 			1},
  	{ "glClearColor",     	gl_ClearColor, 			1},
  	{ "glClearDepth",     	gl_ClearDepth, 			1},
  	{ "glClearIndex",     	gl_ClearIndex, 			1},
  	{ "glClearStencil",   	gl_ClearStencil, 		1},
  	{ "glColor3",        	gl_Color3d, 			1},
  	{ "glColor3b",        	gl_Color3s, 			1},
  	{ "glColor3bv",        	gl_Color3s, 			1},
  	{ "glColor3d",        	gl_Color3d, 			1},
  	{ "glColor3dv",        	gl_Color3d, 			1},
  	{ "glColor3f",        	gl_Color3f, 			1},
  	{ "glColor3fv",        	gl_Color3f, 			1},
  	{ "glColor3i",        	gl_Color3i, 			1},
  	{ "glColor3iv",        	gl_Color3i, 			1},
  	{ "glColor3s",        	gl_Color3s, 			1},
  	{ "glColor3sv",        	gl_Color3s, 			1},
  	{ "glColor3ub",       	gl_Color3s, 			1},
  	{ "glColor3ubv",       	gl_Color3s, 			1},
  	{ "glColor3ui",       	gl_Color3i, 			1},
  	{ "glColor3uiv",       	gl_Color3i, 			1},
  	{ "glColor3us",       	gl_Color3s, 			1},
  	{ "glColor3usv",       	gl_Color3s, 			1},
  	{ "glColor4",       	gl_Color4d, 			1},
  	{ "glColor4b",        	gl_Color4s, 			1},
  	{ "glColor4bv",       	gl_Color4s, 			1},
  	{ "glColor4d",       	gl_Color4d, 			1},
  	{ "glColor4dv",        	gl_Color4d, 			1},
  	{ "glColor4f",        	gl_Color4f, 			1},
  	{ "glColor4fv",        	gl_Color4f, 			1},
  	{ "glColor4i",        	gl_Color4i, 			1},
  	{ "glColor4iv",        	gl_Color4i, 			1},
  	{ "glColor4s",        	gl_Color4s, 			1},
  	{ "glColor4sv",        	gl_Color4s, 			1},
  	{ "glColor4ub",       	gl_Color4s, 			1},
  	{ "glColor4ubv",       	gl_Color4s, 			1},
  	{ "glColor4ui",       	gl_Color4i, 			1},
  	{ "glColor4uiv",       	gl_Color4i, 			1},
  	{ "glColor4us",       	gl_Color4s, 			1},
  	{ "glColor4usv",       	gl_Color4s, 			1},
  	{ "glColorMask",      	gl_ColorMask, 			1},
	{ "glColorMaterial",  	gl_ColorMaterial, 		1},
  	{ "glCopyPixels",     	gl_CopyPixels, 			1},
  	{ "glCullFace",       	gl_CullFace, 			1},
  	{ "glDeleteLists",    	gl_DeleteLists, 		1},
  	{ "glDepthFunc",      	gl_DepthFunc, 			1},
  	{ "glDepthMask",      	gl_DepthMask, 			1},
  	{ "glDepthRange",     	gl_DepthRange, 			1},
  	{ "glDisable",        	gl_Disable, 			1},
  	{ "glDrawBuffer",     	gl_DrawBuffer, 			1},
  	{ "glEdgeFlag",       	gl_EdgeFlag, 			1},
  	{ "glEnable",         	gl_Enable, 			1},
  	{ "glEnd",            	gl_End, 			1},
  	{ "glEndList",        	gl_EndList, 			1},
  	{ "glEvalCoord1",    	gl_EvalCoord1d, 		1},
  	{ "glEvalCoord1d",    	gl_EvalCoord1d, 		1},
 	{ "glEvalCoord1dv",    	gl_EvalCoord1d, 		1},
	{ "glEvalCoord1f",    	gl_EvalCoord1f, 		1},
	{ "glEvalCoord1fv",	gl_EvalCoord1f, 		1},
  	{ "glEvalCoord2",    	gl_EvalCoord2d, 		1},
  	{ "glEvalCoord2d",    	gl_EvalCoord2d, 		1},
 	{ "glEvalCoord2dv",    	gl_EvalCoord2d, 		1},
	{ "glEvalCoord2f",    	gl_EvalCoord2f, 		1},
	{ "glEvalCoord2fv",    	gl_EvalCoord2f, 		1},
  	{ "glEvalMesh1",      	gl_EvalMesh1, 			1},
  	{ "glEvalMesh2",      	gl_EvalMesh2, 			1},
  	{ "glEvalPoint1",     	gl_EvalPoint1, 			1},
  	{ "glEvalPoint2",     	gl_EvalPoint2, 			1},
  	{ "glFinish",         	gl_Finish, 			1},
  	{ "glFlush",          	gl_Flush, 			1},
  	{ "glFog",           	gl_Fogf, 			1},
  	{ "glFogf",           	gl_Fogf, 			1},
	{ "glFogfv",           	gl_Fogf, 			1},
  	{ "glFogi",           	gl_Fogi, 			1},
  	{ "glFogiv",           	gl_Fogi, 			1},
  	{ "glFrontFace",      	gl_FrontFace, 			1},
  	{ "glFrustum",        	gl_Frustum, 			1},
  	{ "glHint",           	gl_Hint, 			1},
  	{ "glIndexMask",      	gl_IndexMask, 			1},
  	{ "glIndex",         	gl_Indexd, 			1},
  	{ "glIndexd",         	gl_Indexd, 			1},
 	{ "glIndexdv",         	gl_Indexd, 			1},
  	{ "glIndexf",         	gl_Indexf, 			1},
  	{ "glIndexfv",         	gl_Indexf, 			1},
  	{ "glIndexi",         	gl_Indexi, 			1},
 	{ "glIndexiv",         	gl_Indexi, 			1},
  	{ "glIndexs",         	gl_Indexs, 			1},
  	{ "glIndexsv",         	gl_Indexs, 			1},
  	{ "glInitNames",      	gl_InitNames, 			1},
  	{ "glLightModel",    	gl_LightModelf, 		1},
  	{ "glLightModelf",    	gl_LightModelf, 		1},
  	{ "glLightModelfv",    	gl_LightModelf, 		1},
  	{ "glLightModeli",    	gl_LightModeli, 		1},
  	{ "glLightModeliv",    	gl_LightModeli, 		1},
  	{ "glLight",         	gl_Lightf, 			1},
  	{ "glLightf",         	gl_Lightf, 			1},
  	{ "glLightfv",        	gl_Lightf, 			1},  
  	{ "glLighti",         	gl_Lighti, 			1},
  	{ "glLightiv",        	gl_Lighti, 			1},  
  	{ "glLineStipple",    	gl_LineStipple, 		1},
  	{ "glLineWidth",      	gl_LineWidth, 			1},
  	{ "glListBase",       	gl_ListBase, 			1},
  	{ "glLoadIdentity",   	gl_LoadIdentity, 		1},
  	{ "glLoadName",       	gl_LoadName, 			1},
  	{ "glLogicOp",        	gl_LogicOp, 			1},
  	{ "glMap1d",      		gl_Map1d, 			1},
  	{ "glMap1f",      		gl_Map1f, 			1},
  	{ "glMap2d",      		gl_Map2d, 			1},
  	{ "glMap2f",      		gl_Map2f, 			1},
  	{ "glMapGrid1d",      	gl_MapGrid1d, 			1},
  	{ "glMapGrid1f",      	gl_MapGrid1f, 			1},
  	{ "glMapGrid2d",      	gl_MapGrid2d, 			1},
  	{ "glMapGrid2f",      	gl_MapGrid2f, 			1},
  	{ "glMaterial",      	gl_Materialf, 			1},
  	{ "glMaterialf",      	gl_Materialf, 			1},
  	{ "glMaterialfv",     	gl_Materialf, 			1},
  	{ "glMateriali",      	gl_Materiali, 			1},
  	{ "glMaterialiv",     	gl_Materiali, 			1},  
  	{ "glMatrixMode",     	gl_MatrixMode, 			1},
  	{ "glMultMatrix",     	gl_MultMatrixd, 		1},
  	{ "glMultMatrixd",     	gl_MultMatrixd, 		1},
  	{ "glMultMatrixf",     	gl_MultMatrixd, 		1},
  	{ "glNewList",        	gl_NewList, 			1},
  	{ "glNormal3",       	gl_Normal3d, 			1},
  	{ "glNormal3b",       	gl_Normal3s, 			1},
  	{ "glNormal3bv",       	gl_Normal3s, 			1},
  	{ "glNormal3d",       	gl_Normal3d, 			1},
  	{ "glNormal3dv",       	gl_Normal3d, 			1},
  	{ "glNormal3f",       	gl_Normal3f, 			1},
  	{ "glNormal3fv",       	gl_Normal3f, 			1},
  	{ "glNormal3i",       	gl_Normal3i, 			1},
 	{ "glNormal3iv",       	gl_Normal3i, 			1},
 	{ "glNormal3s",       	gl_Normal3s, 			1},
 	{ "glNormal3sv",       	gl_Normal3s, 			1},
  	{ "glOrtho",          	gl_Ortho, 			1},
  	{ "glPassThrough",    	gl_PassThrough, 		1},
  	{ "glPixelStoref",    	gl_PixelStoref, 		1},
  	{ "glPixelStorei",    	gl_PixelStorei, 		1},
  	{ "glPixelTransferf", 	gl_PixelTransferf, 		1},
  	{ "glPixelTransferi", 	gl_PixelTransferi, 		1},
  	{ "glPixelZoom",      	gl_PixelZoom, 			1},
  	{ "glPointSize",      	gl_PointSize, 			1},
  	{ "glPolygonMode",    	gl_PolygonMode, 		1},
  	{ "glPopAttrib",      	gl_PopAttrib, 			1},
  	{ "glPopMatrix",      	gl_PopMatrix, 			1},
  	{ "glPopName",        	gl_PopName, 			1},
  	{ "glPushAttrib",     	gl_PushAttrib, 			1},
  	{ "glPushMatrix",     	gl_PushMatrix, 			1},
  	{ "glPushName",       	gl_PushName, 			1},
  	{ "glRasterPos2",    	gl_RasterPos2d, 		1},
  	{ "glRasterPos2d",    	gl_RasterPos2d, 		1},
  	{ "glRasterPos2dv",    	gl_RasterPos2d, 		1},
  	{ "glRasterPos2f",    	gl_RasterPos2f, 		1},
  	{ "glRasterPos2fv",    	gl_RasterPos2f, 		1},
  	{ "glRasterPos2i",    	gl_RasterPos2i, 		1},
  	{ "glRasterPos2iv",    	gl_RasterPos2i, 		1},
  	{ "glRasterPos2s",    	gl_RasterPos2s, 		1},
  	{ "glRasterPos2sv",    	gl_RasterPos2s, 		1},
  	{ "glRasterPos3",    	gl_RasterPos3d, 		1},
  	{ "glRasterPos3d",    	gl_RasterPos3d, 		1},
  	{ "glRasterPos3dv",    	gl_RasterPos3d, 		1},
  	{ "glRasterPos3f",    	gl_RasterPos3f, 		1},
  	{ "glRasterPos3fv",    	gl_RasterPos3f, 		1},
  	{ "glRasterPos3i",    	gl_RasterPos3i, 		1},
  	{ "glRasterPos3iv",    	gl_RasterPos3i, 		1},
  	{ "glRasterPos3s",    	gl_RasterPos3s, 		1},
  	{ "glRasterPos3sv",    	gl_RasterPos3s, 		1},
  	{ "glRasterPos4",    	gl_RasterPos4d, 		1},
  	{ "glRasterPos4d",    	gl_RasterPos4d, 		1},
  	{ "glRasterPos4dv",    	gl_RasterPos4d, 		1},
  	{ "glRasterPos4f",    	gl_RasterPos4f, 		1},
  	{ "glRasterPos4fv",    	gl_RasterPos4f, 		1},
  	{ "glRasterPos4i",    	gl_RasterPos4i, 		1},
  	{ "glRasterPos4iv",    	gl_RasterPos4i, 		1},
  	{ "glRasterPos4s",    	gl_RasterPos4s, 		1},
  	{ "glRasterPos4sv",    	gl_RasterPos4s, 		1},
  	{ "glReadBuffer",     	gl_ReadBuffer, 			1},
  	{ "glRect",          	gl_Rectd,			1},
  	{ "glRectd",          	gl_Rectd,			1},
	{ "glRectdv",          	gl_Rectd,			1},
  	{ "glRectf",          	gl_Rectf,			1},
  	{ "glRectfv",          	gl_Rectf,			1},
  	{ "glRecti",          	gl_Recti, 			1},
  	{ "glRectiv",          	gl_Recti, 			1},
  	{ "glRects",          	gl_Rects, 			1},
  	{ "glRectsv",          	gl_Rects, 			1},
	{ "glRotate",        	gl_Rotated, 			1},
	{ "glRotated",        	gl_Rotated, 			1},
	{ "glRotated",        	gl_Rotated, 			1},
  	{ "glRotatef",        	gl_Rotatef, 			1},
  	{ "glScale",         	gl_Scaled, 			1},
  	{ "glScaled",         	gl_Scaled, 			1},
  	{ "glScalef",         	gl_Scalef,			1},
  	{ "glScissor",        	gl_Scissor,			1},
  	{ "glShadeModel",     	gl_ShadeModel,			1},
  	{ "glStencilFunc",    	gl_StencilFunc,			1},
  	{ "glStencilMask",    	gl_StencilMask,			1},
  	{ "glStencilOp",      	gl_StencilOp,			1},
  	{ "glTexCoord1",     	gl_TexCoord1d, 			1},
  	{ "glTexCoord1d",     	gl_TexCoord1d, 			1},
 	{ "glTexCoord1dv",     	gl_TexCoord1d, 			1},
  	{ "glTexCoord1f",     	gl_TexCoord1f,			1},
  	{ "glTexCoord1fv",     	gl_TexCoord1f,			1},
  	{ "glTexCoord1i",     	gl_TexCoord1i,			1},
  	{ "glTexCoord1iv",     	gl_TexCoord1i,			1},
  	{ "glTexCoord1s",     	gl_TexCoord1s,			1},
  	{ "glTexCoord1sv",     	gl_TexCoord1s,			1},
 	{ "glTexCoord2",     	gl_TexCoord2d, 			1},
 	{ "glTexCoord2d",     	gl_TexCoord2d, 			1},
 	{ "glTexCoord2dv",     	gl_TexCoord2d, 			1},
  	{ "glTexCoord2f",     	gl_TexCoord2f,			1},
  	{ "glTexCoord2fv",     	gl_TexCoord2f,			1},
  	{ "glTexCoord2i",     	gl_TexCoord2i,			1},
  	{ "glTexCoord2iv",     	gl_TexCoord2i,			1},
  	{ "glTexCoord2s",     	gl_TexCoord2s,			1},
  	{ "glTexCoord2sv",     	gl_TexCoord2s,			1},
 	{ "glTexCoord3",     	gl_TexCoord3d, 			1},
 	{ "glTexCoord3d",     	gl_TexCoord3d, 			1},
 	{ "glTexCoord3dv",     	gl_TexCoord3d, 			1},
  	{ "glTexCoord3f",     	gl_TexCoord3f,			1},
  	{ "glTexCoord3fv",     	gl_TexCoord3f,			1},
  	{ "glTexCoord3i",     	gl_TexCoord3i,			1},
  	{ "glTexCoord3iv",     	gl_TexCoord3i,			1},
  	{ "glTexCoord3s",     	gl_TexCoord3s,			1},
  	{ "glTexCoord3sv",     	gl_TexCoord3s,			1},
 	{ "glTexCoord4",     	gl_TexCoord4d, 			1},
 	{ "glTexCoord4d",     	gl_TexCoord4d, 			1},
 	{ "glTexCoord4dv",     	gl_TexCoord4d, 			1},
  	{ "glTexCoord4f",     	gl_TexCoord4f,			1},
  	{ "glTexCoord4fv",     	gl_TexCoord4f,			1},
  	{ "glTexCoord4i",     	gl_TexCoord4i,			1},
  	{ "glTexCoord4iv",     	gl_TexCoord4i,			1},
  	{ "glTexCoord4s",     	gl_TexCoord4s,			1},
  	{ "glTexCoord4sv",     	gl_TexCoord4s,			1},
  	{ "glTexEnv",        	gl_TexEnvf, 			1},
  	{ "glTexEnvf",        	gl_TexEnvf, 			1},
  	{ "glTexEnvfv",        	gl_TexEnvf, 			1},
  	{ "glTexEnvi",        	gl_TexEnvi,			1},
 	{ "glTexEnviv",        	gl_TexEnvi,			1},
  	{ "glTexGen",        	gl_TexGend, 			1},
  	{ "glTexGend",        	gl_TexGend, 			1},
  	{ "glTexGendv",        	gl_TexGend, 			1},
  	{ "glTexGenf",        	gl_TexGenf,			1},
  	{ "glTexGenfv",        	gl_TexGenf, 			1},
  	{ "glTexGeni",        	gl_TexGeni,			1},
  	{ "glTexGeniv",        	gl_TexGeni, 			1},
  	{ "glTexParameter",  	gl_TexParameterf, 		1},
  	{ "glTexParameterf",  	gl_TexParameterf, 		1},
  	{ "glTexParameterfv",  	gl_TexParameterf, 		1},
  	{ "glTexParameteri",  	gl_TexParameteri,		1},
  	{ "glTexParameteriv",  	gl_TexParameteri, 		1},
  	{ "glTranslate",     	gl_Translated, 			1},
  	{ "glTranslated",     	gl_Translated, 			1},
  	{ "glTranslatef",     	gl_Translatef,			1},
  	{ "glVertex2",       	gl_Vertex2d, 			1},
  	{ "glVertex2d",       	gl_Vertex2d, 			1},
  	{ "glVertex2dv",      	gl_Vertex2d, 			1},
  	{ "glColorVertex2",     	gl_ColorVertex2d, 		1},
  	{ "glColorVertex2d",    	gl_ColorVertex2d, 		1},
  	{ "glVertex2f",       	gl_Vertex2f,			1},
  	{ "glVertex2fv",      	gl_Vertex2f, 			1},
  	{ "glVertex2i",       	gl_Vertex2i,			1},
  	{ "glVertex2iv",       	gl_Vertex2i, 			1},
#ifdef NUMERIC
	{ "glVertex2s",       	gl_Vertex2s, 			1},
	{ "glVertex2sv",       	gl_Vertex2s, 			1},
#endif
  	{ "glVertex3",       	gl_Vertex3d, 			1},
  	{ "glVertex3d",       	gl_Vertex3d, 			1},
  	{ "glVertex3dv",      	gl_Vertex3d, 			1},
  	{ "glVertex3f",       	gl_Vertex3f,			1},
  	{ "glVertex3fv",      	gl_Vertex3f, 			1},
  	{ "glVertex3i",       	gl_Vertex3i,			1},
  	{ "glVertex3iv",       	gl_Vertex3i, 			1},
  	{ "glVertex3s",       	gl_Vertex3s, 			1},
  	{ "glVertex3sv",       	gl_Vertex3s, 			1},
  	{ "glVertex4",       	gl_Vertex4d, 			1},
  	{ "glVertex4d",       	gl_Vertex4d, 			1},
  	{ "glVertex4dv",      	gl_Vertex4d, 			1},
  	{ "glVertex4f",       	gl_Vertex4f,			1},
  	{ "glVertex4fv",      	gl_Vertex4f, 			1},
  	{ "glVertex4i",       	gl_Vertex4i,			1},
  	{ "glVertex4iv",       	gl_Vertex4i, 			1},
  	{ "glVertex4s",       	gl_Vertex4s, 			1},
  	{ "glVertex4sv",       	gl_Vertex4s, 			1},
  	{ "glViewport",       	gl_Viewport, 			1},
 
 	{ "glClipPlane",		gl_ClipPlane,			1},   
 	{ "glGetClipPlane",	gl_GetClipPlane,		1},   
 	{ "glIsEnabled",		gl_IsEnabled,			1},   
 	{ "glIsList",		gl_IsList,			1},   
 	{ "glRenderMode",		gl_RenderMode,			1},
	{ "glGenLists",		gl_GenLists,			1}, 
  	{ "glGetDouble",		gl_GetDoublev, 			1},  
  	{ "glGetDoublev",		gl_GetDoublev, 			1},

/*Not terribly useful without NumPy, but they'll work*/
	{ "glDrawPixels",		gl_DrawPixels,			1},  
#ifdef NUMERIC
	{ "glTexImage2D",		gl_TexImage2D,			1},
	{ "glSelectBuffer",	        gl_SelectBuffer,	1}, 
#endif /* NUMERIC */
	{ "glGetError",	                gl_GetError,	        1}, 
	{ "glReadPixels",		gl_ReadPixels,			1},
	
/*Various OpenGL extensions */
  	{ "glPolygonOffsetEXT", 	gl_PolygonOffsetEXT,		1},
  	{ "glArrayElementEXT",	gl_ArrayElementEXT,		1}, 
#ifdef NOT_DEFINED
	{ "glDrawArraysEXT",	gl_DrawArraysEXT,		1},  
 	{ "glBlendColorEXT",	gl_BlendColorEXT,		1},   
 	{ "glBlendEquationEXT",	gl_BlendEquationEXT,		1},  
#endif

#ifdef OPENGL1_1 
#ifdef NUMERIC
  	{ "glVertexPointer",		gl_VertexPointer,	1},
  	{ "glColorPointer",		gl_ColorPointer,	1},
  	{ "glDrawArrays",			gl_DrawArrays,		1}, 
#endif /* NUMERIC */
#endif /* OPENGL1_1 */
  	{NULL,               NULL} 
};

#ifdef WIN32
__declspec(dllexport)
#endif
#ifdef NUMERIC
void init_opengl_num()
#else
void init_opengl()
#endif
{
	PyObject *m, *d;
	PyObject *nl, *gl;
#ifdef NUMERIC
	m = Py_InitModule("_opengl_num", gl_methods);
#ifdef import_array  /* Konrad Hinsen's version */
	import_array();
#endif
#else
	m = Py_InitModule("_opengl", gl_methods);
#endif
	d = PyModule_GetDict(m);
#ifdef NUMERIC
	gl_Error = Py_BuildValue("s", "_opengl_num.error");
#else
	gl_Error = Py_BuildValue("s", "_opengl.error");
#endif
	PyDict_SetItemString(d, "error", gl_Error);
#ifdef NUMERIC
	nl = PyInt_FromLong(1L);
#else
	nl = PyInt_FromLong(0L);
#endif
	PyDict_SetItemString(d, "_numeric", nl);
	Py_DECREF(nl);
#ifdef GLUT
	gl = PyInt_FromLong(1L);
#else
	gl = PyInt_FromLong(0L);
#endif
	PyDict_SetItemString(d, "_glut", gl);
	Py_DECREF(gl);
	if (PyErr_Occurred()) 
#ifdef NUMERIC
		Py_FatalError("can't initialize module _opengl_num");
#else
		Py_FatalError("can't initialize module _opengl");
#endif
}
