/*
* Simple interface to libxslt
*
*   Copyright (c) 2001 Dave Kuhlman
*   
*   Permission is hereby granted, free of charge, to any person obtaining
*   a copy of this software and associated documentation files (the
*   "Software"), to deal in the Software without restriction, including
*   without limitation the rights to use, copy, modify, merge, publish,
*   distribute, sublicense, and/or sell copies of the Software, and to
*   permit persons to whom the Software is furnished to do so, subject to
*   the following conditions:
*   
*   The above copyright notice and this permission notice shall be
*   included in all copies or substantial portions of the Software.
*   
*   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
*   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
*   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
*   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
*   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
*   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
*/


#include <libxml/xmlmemory.h>
#include <libxml/debugXML.h>
#include <libxml/HTMLtree.h>
#include <libxml/xmlIO.h>
#include <libxml/DOCBparser.h>
#include <libxml/xinclude.h>
#include <libxml/catalog.h>
#include <libxml/parser.h>
#include <libxml/parserInternals.h>

#include <libxslt/xslt.h>
#include <libxslt/xsltInternals.h>
#include <libxslt/transform.h>
#include <libxslt/xsltutils.h>
#include <libxslt/imports.h>

#include <Python.h>

#include "libxslt_stylesheet.h"

/* ------------------------------------------- */

extern int xmlLoadExtDtdDefaultValue;

/* ------------------------------------------- */

typedef struct _UserData {
	PyObject * messageHandler;
} UserData, * UserDataPtr;

/* ------------------------------------------- */

/* Globals:
*/

/* ------------------------------------------- */

void formatFileInfo(
	xmlParserInputPtr input,
	char * buffer
)
{
	buffer[0] = '\0';
	if (input != NULL)
	{
		if (input->filename)
		{
			sprintf(buffer, "%s:%d", input->filename, input->line);
		}
		else
		{
			sprintf(buffer, "Entity: line %d", input->line);
		} /* if */
	} /* if */
} /* formatFileInfo */

/* ------------------------------------------- */

/* Format an error message and deliver it to the (Python) message
*   handler.
* Here is a sample of the format:
*
* camping_format_error.xsl:28: error: Opening and ending tag mismatch: xsl:template and xsl:templateXXXX
*
*/

static void handleMessages(
	void *user_data,
	const char * msg,
	...
)
{
	PyObject * method;
	PyObject * arglist;
	PyObject * result;
    va_list args;
    char buf[1024];
	char buf1[1024];
	char buf2[1024];
	UserData * userData;
	xmlParserCtxt * ctxt;
	PyObject * messageHandler;

	ctxt = (xmlParserCtxt *)user_data;
	userData = (UserData *)ctxt->_private;
	messageHandler = userData->messageHandler;
	method = PyObject_GetAttrString(messageHandler, "write");
	if (method != NULL)
	{
		formatFileInfo(ctxt->input, buf1);
		va_start(args, msg);
		vsprintf(buf2, msg, args);
		sprintf(buf, "%s:%s", buf1, buf2);
		arglist = Py_BuildValue("(s)", buf);
		result = PyEval_CallObject(method, arglist);
		Py_DECREF(arglist);
	} /* if */
} /* handleMessages */

/* ------------------------------------------- */

#if 0

xmlSAXHandler messageSAXHandler = {
	0,		/*    internalSubsetSAXFunc internalSubset; */
	0,		/*    isStandaloneSAXFunc isStandalone; */
	0,		/*    hasInternalSubsetSAXFunc hasInternalSubset; */
	0,		/*    hasExternalSubsetSAXFunc hasExternalSubset; */
	0,		/*    resolveEntitySAXFunc resolveEntity; */
	0,		/*    getEntitySAXFunc getEntity; */
	0,		/*    entityDeclSAXFunc entityDecl; */
	0,		/*    notationDeclSAXFunc notationDecl; */
	0,		/*    attributeDeclSAXFunc attributeDecl; */
	0,		/*    elementDeclSAXFunc elementDecl; */
	0,		/*    unparsedEntityDeclSAXFunc unparsedEntityDecl; */
	0,		/*    setDocumentLocatorSAXFunc setDocumentLocator; */
	0,		/*    startDocumentSAXFunc startDocument; */
	0,		/*    endDocumentSAXFunc endDocument; */
	0,		/*    startElementSAXFunc startElement; */
	0,		/*    endElementSAXFunc endElement; */
	0,		/*    referenceSAXFunc reference; */
	0,		/*    charactersSAXFunc characters; */
	0,		/*    ignorableWhitespaceSAXFunc ignorableWhitespace; */
	0,		/*    processingInstructionSAXFunc processingInstruction; */
	0,		/*    commentSAXFunc comment; */
	handleMessages,		/*    warningSAXFunc warning; */
	handleMessages,		/*    errorSAXFunc error; */
	handleMessages,		/*    fatalErrorSAXFunc fatalError; */
	0,		/*    getParameterEntitySAXFunc getParameterEntity; */
	0,		/*    cdataBlockSAXFunc cdataBlock; */
	0,		/*    externalSubsetSAXFunc externalSubset; */
};

#endif

/* ------------------------------------------- */

typedef struct _StringOutputBuffer 
{
	int len;
	int increment;
	int pos;
	char * buffer;
} StringOutputBuffer, * StringOutputBufferPtr;

static int
outputBufferWrite(
	StringOutputBuffer * obuf,
	const char * buffer,
	int len
)
{
	if ((obuf->pos + len) >= obuf->len)
	{
		obuf->buffer = realloc(obuf->buffer, obuf->len + obuf->increment);
	} /* if */
	memmove(&obuf->buffer[obuf->pos], &buffer[0], len);
	obuf->pos += len;
	return 1;
} /* outputBufferWrite */


static int
outputBufferClose(
	StringOutputBuffer * obuf
)
{
	return 1;
} /* outputBufferClose */

/* ------------------------------------------- */

typedef struct _StreamOutputBuffer 
{
	PyObject * streamObject;
	char * buffer;
	int len;
} StreamOutputBuffer, * StreamOutputBufferPtr;

static int
streamOutputBufferWrite(
	StreamOutputBuffer * obuf,
	const char * buffer,
	int len
)
{
	PyObject * result;

	if ((obuf->buffer == NULL) || len >= obuf->len)
	{
		obuf->buffer  = (char *)malloc(len * 2);
		obuf->len = len * 2;
	} /* if */
	strncpy(obuf->buffer, buffer, len);
	obuf->buffer[len] = '\0';
	
	result = PyObject_CallMethod(obuf->streamObject, "write",
		"s", obuf->buffer);
	if (PyErr_Occurred())
	{
		PyErr_Print();
	} /* if */

	return 1;
} /* streamOutputBufferWrite */


static int
streamOutputBufferClose(
	StreamOutputBuffer * obuf
)
{
	return 1;
} /* streamOutputBufferClose */

/* ------------------------------------------- */


/* This is a copy of libxml/parser.c/xmlSAXParseFile, with
*	a bit added to override the error handlers.
*/

/**
 * xmlSAXParseFile:
 * @sax:  the SAX handler block
 * @filename:  the filename
 * @recovery:  work in recovery mode, i.e. tries to read no Well Formed
 *             documents
 *
 * parse an XML file and build a tree. Automatic support for ZLIB/Compress
 * compressed document is provided by default if found at compile-time.
 * It use the given SAX function block to handle the parsing callback.
 * If sax is NULL, fallback to the default DOM tree building routines.
 *
 * Returns the resulting document tree
 */

xmlDocPtr
SAXParseFile(
	const char *filename,
	int recovery,
	PyObject * messageHandler
)
{
	xmlDocPtr ret;
	xmlParserCtxtPtr ctxt;
	char *directory = NULL;
	UserData * userData;
	UserData user_data;

	ctxt = xmlCreateFileParserCtxt(filename);
	if (ctxt == NULL) {
		return(NULL);
	}

	if (messageHandler != Py_None)
	{	
		if (ctxt->sax != NULL)
		{
			ctxt->sax->warning = handleMessages;
			ctxt->sax->error = handleMessages;
			ctxt->sax->fatalError = handleMessages;
			userData = &user_data;
			userData->messageHandler = messageHandler;
			ctxt->_private = userData;
	    } /* if */
	} /* if */

	if ((ctxt->directory == NULL) && (directory == NULL))
		directory = xmlParserGetDirectory(filename);
	if ((ctxt->directory == NULL) && (directory != NULL))
		ctxt->directory = (char *) xmlStrdup((xmlChar *) directory);

	xmlParseDocument(ctxt);

	if ((ctxt->wellFormed) || recovery)
	{
		ret = ctxt->myDoc;
	}
	else
	{
	   ret = NULL;
	   xmlFreeDoc(ctxt->myDoc);
	   ctxt->myDoc = NULL;
	} /* if */

	ctxt->sax = NULL;
	xmlFreeParserCtxt(ctxt);
	
	return(ret);
} /* SAXParseFile */

/* ------------------------------------------- */

/* This is a copy of libxml/parser.c/xmlSAXParseMemory, with
*	a bit added to override the error handlers.
*/

/**
 * xmlSAXParseMemory:
 * @sax:  the SAX handler block
 * @buffer:  an pointer to a char array
 * @size:  the size of the array
 * @recovery:  work in recovery mode, i.e. tries to read not Well Formed
 *             documents
 *
 * parse an XML in-memory block and use the given SAX function block
 * to handle the parsing callback. If sax is NULL, fallback to the default
 * DOM tree building routines.
 * 
 * Returns the resulting document tree
 */
xmlDocPtr
SAXParseMemory(
	char *buffer,
	int size,
	int recovery,
	PyObject * messageHandler
)
{
	xmlDocPtr ret;
	xmlParserCtxtPtr ctxt;
	UserData * userData;
	UserData user_data;

	ctxt = xmlCreateMemoryParserCtxt(buffer, size);
	if (ctxt == NULL)
	{
		return(NULL);
	} /* if */

	if (messageHandler != Py_None)
	{	
		if (ctxt->sax != NULL)
		{
			ctxt->sax->warning = handleMessages;
			ctxt->sax->error = handleMessages;
			ctxt->sax->fatalError = handleMessages;
			userData = &user_data;
			userData->messageHandler = messageHandler;
			ctxt->_private = userData;
	    } /* if */
	} /* if */

	xmlParseDocument(ctxt);

	if ((ctxt->wellFormed) || recovery)
	{
		ret = ctxt->myDoc;
	}
	else
	{
	   ret = NULL;
	   xmlFreeDoc(ctxt->myDoc);
	   ctxt->myDoc = NULL;
	} /* if */

	ctxt->sax = NULL;
	xmlFreeParserCtxt(ctxt);

	return(ret);
}

/* ------------------------------------------- */

/*
* Translate doc and return result as Python string.
* Interface:
	translate_to_string(
		stylesheetType		-- 's' for string; 'f' for filename, 'c' for compiled
		stylesheet			-- stylesheet string or filename
		inDocumentType		-- 's' for string; 'f' for filename
		inDocument			-- input document string or filename
*/



static PyObject * translate_to_string(
	PyObject * self,
	PyObject * args
)
{
	char * stylesheetType;
	char * stylesheetSource;
	char * inType;
	char * inSource;
	xmlDocPtr stylesheetDoc;
	xsltStylesheetPtr stylesheet;
	xmlDocPtr inDoc;
	xmlDocPtr resultDoc;
	xmlOutputBufferPtr outputBuf;
	xmlCharEncodingHandlerPtr encoder;
    const xmlChar *encoding;
	StringOutputBuffer * obuf;
	PyObject * pyStylesheet;
	Libxslt_stylesheetObject * stylesheetobject;
	PyObject * messageHandler;
	PyObject * method;
	PyObject * result;

	pyStylesheet = NULL;
	if (! PyArg_ParseTuple(args, "ssssO", &stylesheetType, &stylesheetSource,
			&inType, &inSource, &messageHandler))
	{
		PyErr_Clear();
		if (! PyArg_ParseTuple(args, "sOssO",
				&stylesheetType, &pyStylesheet,
				&inType, &inSource, &messageHandler))
		{
			PyErr_SetString(PyExc_TypeError, "usage: (str, str, str, str, {object|None}) or (str, tuple, str, str, {object|None})");
			return NULL;
		} /* if */
	} /* if */

	if (messageHandler != Py_None)
	{
		if (! PyInstance_Check(messageHandler))
		{
			PyErr_SetString(PyExc_TypeError, "message handler must be and instance of a class");
			return NULL;
		} /* if */

		method = PyObject_GetAttrString(messageHandler, "write");
		if (method == NULL)
		{
			PyErr_SetString(PyExc_TypeError, "message handler must have a 'write' method");
			return NULL;
		} /* if */
	} /* if */

	xmlSubstituteEntitiesDefault(1);
	xmlLoadExtDtdDefaultValue = 1;

	stylesheet = NULL;
	stylesheetDoc = NULL;
	switch (stylesheetType[0])
	{
	case 'f':
		stylesheetDoc = SAXParseFile(
			(const xmlChar *)stylesheetSource, 0, messageHandler);
		if (stylesheetDoc == NULL)
		{
			PyErr_SetString(PyExc_RuntimeError, "Can't parse stylesheet");
			return NULL;
		} /* if */
		stylesheet = xsltParseStylesheetDoc(stylesheetDoc);
		break;
	case 's':
		stylesheetDoc = SAXParseMemory(
			(xmlChar *)stylesheetSource,
			strlen(stylesheetSource), 0, messageHandler);
		if (stylesheetDoc == NULL)
		{
			PyErr_SetString(PyExc_RuntimeError, "Can't parse stylesheet");
			return NULL;
		} /* if */
		stylesheet = xsltParseStylesheetDoc(stylesheetDoc);
		break;
	case 'c':
		if (pyStylesheet == NULL)
		{
			PyErr_SetString(PyExc_RuntimeError, "'c' option requires compiled stylesheet object");
			return NULL;
		} /* if */
		stylesheetobject = (Libxslt_stylesheetObject *)pyStylesheet;
		if (strcmp(stylesheetobject->ob_type->tp_name, LIBXSLT_STYLESHEET_NAME) != 0)
		{
			PyErr_SetString(PyExc_RuntimeError, "'c' option requires compiled stylesheet object");
			return NULL;
		} /* if */
		stylesheet = stylesheetobject->obj_xsltStylesheetPtr;
		break;
	default:
		break;
	} /* switch */
	

	inDoc = NULL;
	switch (inType[0])
	{
	case 'f':
		inDoc = SAXParseFile((const xmlChar *)inSource, 0, messageHandler);
		break;
	case 's':
		inDoc = SAXParseMemory((xmlChar *)inSource,
			strlen(inSource), 0, messageHandler);
		break;
	default:
		break;
	} /* switch */
	
	if (inDoc == NULL)
	{
		PyErr_SetString(PyExc_RuntimeError, "Can't parse source XML doc");
		return NULL;
	} /* if */

	resultDoc = xsltApplyStylesheet(stylesheet, inDoc, NULL);

	encoder = NULL;

	XSLT_GET_IMPORT_PTR(encoding, stylesheet, encoding)
	if (encoding != NULL)
	{

		encoder = xmlFindCharEncodingHandler((char *)encoding);
		if ((encoder != NULL) &&
			(xmlStrEqual((const xmlChar *)encoder->name,
						 (const xmlChar *) "UTF-8")))
		{
			encoder = NULL;
		} /* if */
	} /* if */

	obuf = malloc(sizeof(StringOutputBuffer));
	obuf->increment = 4096;
	obuf->buffer = malloc(4096);
	obuf->len = 4096;
	obuf->pos = 0;
	outputBuf = xmlOutputBufferCreateIO(
		(xmlOutputWriteCallback)outputBufferWrite,
		(xmlOutputCloseCallback)outputBufferClose,
		(void *)obuf,
		encoder);
	xsltSaveResultTo(outputBuf, resultDoc, stylesheet);
	result = PyString_FromStringAndSize(obuf->buffer, obuf->pos);
	
	if (stylesheetType[0] != 'c')
	{
		xsltFreeStylesheet(stylesheet);
		/* Causes crash
		xmlFreeDoc(stylesheetDoc);
		*/
	} /* if */
	xmlFreeDoc(inDoc);
	xmlFreeDoc(resultDoc);

	return result;
} /* translate_to_string */

/* ------------------------------------------- */

/*
* Translate doc and write the result to a Python file object,
* i.e. to an object that has a 'write' method.
* Interface:
	translate_to_string(
		stylesheetType		-- 's' for string; 'f' for filename, 'c' for compiled
		stylesheet			-- stylesheet string or filename
		inDocumentType		-- 's' for string; 'f' for filename
		inDocument			-- input document string or filename
*/



static PyObject * translate_to_stream(
	PyObject * self,
	PyObject * args
)
{
	char * stylesheetType;
	char * stylesheetSource;
	char * inType;
	char * inSource;
	xmlDocPtr stylesheetDoc;
	xsltStylesheetPtr stylesheet;
	xmlDocPtr inDoc;
	xmlDocPtr resultDoc;
	xmlOutputBufferPtr outputBuf;
	xmlCharEncodingHandlerPtr encoder;
    const xmlChar *encoding;
	StreamOutputBuffer * obuf;
	PyObject * pyStylesheet;
	Libxslt_stylesheetObject * stylesheetobject;
	PyObject * messageHandler;
	PyObject * method;
	PyObject * pyStream;
	PyObject * writeMethod;
	PyObject * result;

	pyStylesheet = NULL;
	pyStream = NULL;
	if (! PyArg_ParseTuple(args, "ssssOO:translate_to_stream",
			&stylesheetType, &stylesheetSource,
			&inType, &inSource,
			&messageHandler, &pyStream))
	{
		PyErr_Clear();
		if (! PyArg_ParseTuple(args, "sOssOO:translate_to_stream",
				&stylesheetType, &pyStylesheet,
				&inType, &inSource,
				&messageHandler, &pyStream))
		{
			PyErr_SetString(PyExc_TypeError, "usage: (str, str, str, str, {object|None}) or (str, tuple, str, str, {object|None})");
			return NULL;
		} /* if */
	} /* if */

	if (messageHandler != Py_None)
	{
		if (! PyInstance_Check(messageHandler))
		{
			PyErr_SetString(PyExc_TypeError, "message handler must be and instance of a class");
			return NULL;
		} /* if */

		method = PyObject_GetAttrString(messageHandler, "write");
		if (method == NULL || (! PyMethod_Check(method)))
		{
			PyErr_SetString(PyExc_TypeError, "message handler must have a 'write' method");
			return NULL;
		} /* if */
	} /* if */

	if (pyStream != Py_None)
	{
		writeMethod = PyObject_GetAttrString(pyStream, "write");
		if (writeMethod == NULL || (! PyCallable_Check(writeMethod)))
		{
			PyErr_SetString(PyExc_TypeError, "stream must have a 'write' method");
			return NULL;
		} /* if */
	} /* if */

	xmlSubstituteEntitiesDefault(1);
	xmlLoadExtDtdDefaultValue = 1;

	stylesheet = NULL;
	stylesheetDoc = NULL;
	switch (stylesheetType[0])
	{
	case 'f':
		stylesheetDoc = SAXParseFile(
			(const xmlChar *)stylesheetSource, 0, messageHandler);
		if (stylesheetDoc == NULL)
		{
			PyErr_SetString(PyExc_RuntimeError, "Can't parse stylesheet");
			return NULL;
		} /* if */
		stylesheet = xsltParseStylesheetDoc(stylesheetDoc);
		break;
	case 's':
		stylesheetDoc = SAXParseMemory(
			(xmlChar *)stylesheetSource,
			strlen(stylesheetSource), 0, messageHandler);
		if (stylesheetDoc == NULL)
		{
			PyErr_SetString(PyExc_RuntimeError, "Can't parse stylesheet");
			return NULL;
		} /* if */
		stylesheet = xsltParseStylesheetDoc(stylesheetDoc);
		break;
	case 'c':
		if (pyStylesheet == NULL)
		{
			PyErr_SetString(PyExc_RuntimeError, "'c' option requires compiled stylesheet object");
			return NULL;
		} /* if */
		stylesheetobject = (Libxslt_stylesheetObject *)pyStylesheet;
		if (strcmp(stylesheetobject->ob_type->tp_name, LIBXSLT_STYLESHEET_NAME) != 0)
		{
			PyErr_SetString(PyExc_RuntimeError, "'c' option requires compiled stylesheet object");
			return NULL;
		} /* if */
		stylesheet = stylesheetobject->obj_xsltStylesheetPtr;
		break;
	default:
		break;
	} /* switch */
	

	inDoc = NULL;
	switch (inType[0])
	{
	case 'f':
		inDoc = SAXParseFile((const xmlChar *)inSource, 0, messageHandler);
		break;
	case 's':
		inDoc = SAXParseMemory((xmlChar *)inSource,
			strlen(inSource), 0, messageHandler);
		break;
	default:
		break;
	} /* switch */
	
	if (inDoc == NULL)
	{
		PyErr_SetString(PyExc_RuntimeError, "Can't parse source XML doc");
		return NULL;
	} /* if */

	resultDoc = xsltApplyStylesheet(stylesheet, inDoc, NULL);

	encoder = NULL;

	XSLT_GET_IMPORT_PTR(encoding, stylesheet, encoding)
	if (encoding != NULL)
	{

		encoder = xmlFindCharEncodingHandler((char *)encoding);
		if ((encoder != NULL) &&
			(xmlStrEqual((const xmlChar *)encoder->name,
						 (const xmlChar *) "UTF-8")))
		{
			encoder = NULL;
		} /* if */
	} /* if */

	obuf = malloc(sizeof(StreamOutputBuffer));
	obuf->streamObject = pyStream;
	obuf->buffer = NULL;
	obuf->len = 0;
	outputBuf = xmlOutputBufferCreateIO(
		(xmlOutputWriteCallback)streamOutputBufferWrite,
		(xmlOutputCloseCallback)streamOutputBufferClose,
		(void *)obuf,
		encoder);
	xsltSaveResultTo(outputBuf, resultDoc, stylesheet);
	if (obuf->buffer != NULL)
	{
		free(obuf->buffer);
	} /* if */

	if (stylesheetType[0] != 'c')
	{
		xsltFreeStylesheet(stylesheet);
		/* Causes crash
		xmlFreeDoc(stylesheetDoc);
		*/
	} /* if */
	xmlFreeDoc(inDoc);
	xmlFreeDoc(resultDoc);

	Py_INCREF(Py_None);
	result =  Py_None;
	return result;
} /* translate_to_stream */

/* ------------------------------------------- */

/*
* Translate doc and write result to file.
* Interface:
	translate_to_file(
		stylesheetType,		-- 's' for string; 'f' for filename, 'c' for compiled
		stylesheet,			-- stylesheet string or filename
		inDocumentType,		-- 's' for string; 'f' for filename
		inDocument,			-- input document string or filename
		outfileName,		-- name of file to write result to
		messageHandler		-- an instance of a handler class or None
	)
*/

static PyObject * translate_to_file(
	PyObject * self,
	PyObject * args
)
{
	char * stylesheetType;
	char * stylesheetSource;
	char * inType;
	char * inSource;
	char * outputFileName;
	xmlDocPtr stylesheetDoc;
	xsltStylesheetPtr stylesheet;
	xmlDocPtr inDoc;
	xmlDocPtr resultDoc;
	FILE * outFile;
	PyObject * pyStylesheet;
	Libxslt_stylesheetObject * stylesheetobject;
	PyObject * messageHandler;
	PyObject * method;
	PyObject * result;

	pyStylesheet = NULL;
	if (! PyArg_ParseTuple(args, "sssssO", &stylesheetType, &stylesheetSource,
			&inType, &inSource, &outputFileName, &messageHandler))
	{
		PyErr_Clear();
		if (! PyArg_ParseTuple(args, "sOsssO",
				&stylesheetType, &pyStylesheet,
				&inType, &inSource, &outputFileName, &messageHandler))
		{
			PyErr_SetString(PyExc_TypeError, "usage: (str, {str|object}, str, str, str {object|None})");
			return NULL;
		} /* if */
	} /* if */

	if (messageHandler != Py_None)
	{
		if (! PyInstance_Check(messageHandler))
		{
			PyErr_SetString(PyExc_TypeError, "message handler must be and instance of a class");
			return NULL;
		} /* if */

		method = PyObject_GetAttrString(messageHandler, "write");
		if (method == NULL)
		{
			PyErr_SetString(PyExc_TypeError, "message handler must have a 'write' method");
			return NULL;
		} /* if */
	} /* if */

	xmlSubstituteEntitiesDefault(1);
	xmlLoadExtDtdDefaultValue = 1;

	stylesheet = NULL;
	stylesheetDoc = NULL;
	switch (stylesheetType[0])
	{
	case 'f':
		stylesheetDoc = SAXParseFile(
			(const xmlChar *)stylesheetSource, 0, messageHandler);
		if (stylesheetDoc == NULL)
		{
			PyErr_SetString(PyExc_RuntimeError, "Can't parse stylesheet");
			return NULL;
		} /* if */
		stylesheet = xsltParseStylesheetDoc(stylesheetDoc);
		break;
	case 's':
		stylesheetDoc = SAXParseMemory(
			(xmlChar *)stylesheetSource,
			strlen(stylesheetSource), 0, messageHandler);
		if (stylesheetDoc == NULL)
		{
			PyErr_SetString(PyExc_RuntimeError, "Can't parse stylesheet");
			return NULL;
		} /* if */
		stylesheet = xsltParseStylesheetDoc(stylesheetDoc);
		break;
	case 'c':
		if (pyStylesheet == NULL)
		{
			PyErr_SetString(PyExc_RuntimeError, "'c' option requires compiled stylesheet object");
			return NULL;
		} /* if */
		stylesheetobject = (Libxslt_stylesheetObject *)pyStylesheet;
		if (strcmp(stylesheetobject->ob_type->tp_name, LIBXSLT_STYLESHEET_NAME) != 0)
		{
			PyErr_SetString(PyExc_RuntimeError, "'c' option requires compiled stylesheet object");
			return NULL;
		} /* if */

		stylesheet = stylesheetobject->obj_xsltStylesheetPtr;

		break;
	default:
		break;
	} /* switch */

	inDoc = NULL;
	switch (inType[0])
	{
	case 'f':
		inDoc = SAXParseFile((const xmlChar *)inSource, 0, messageHandler);
		break;
	case 's':
		inDoc = SAXParseMemory((xmlChar *)inSource,
			strlen(inSource), 0, messageHandler);
		break;
	default:
		break;
	} /* switch */
	
	if (inDoc == NULL)
	{
		PyErr_SetString(PyExc_RuntimeError, "Can't parse source XML doc");
		return NULL;
	} /* if */

	resultDoc = xsltApplyStylesheet(stylesheet, inDoc, NULL);
	outFile = fopen(outputFileName, "w");
	xsltSaveResultToFile(outFile, resultDoc, stylesheet);
	fclose(outFile);
	xmlFreeDoc(resultDoc);

	xmlFreeDoc(inDoc);
	if (stylesheetType[0] != 'c')
	{
		xsltFreeStylesheet(stylesheet);
		/* Causes crash
		xmlFreeDoc(stylesheetDoc);
		*/
	} /* if */

	result = PyInt_FromLong((long)1);
	return result;
} /* translate_to_file */

/* ------------------------------------------- */

/*
* Compile a stylesheet.
* Return it for use with translate_to_file and translate_to_string
*   using the 'c' option.
* Interface:
	compile_stylesheet(
		stylesheetType		-- 's' for string; 'f' for filename
		stylesheet)			-- stylesheet string or filename
*/

static PyObject * compile_stylesheet(
	PyObject * self,
	PyObject * args
)
{
	char * stylesheetType;
	char * stylesheetSource;
	xmlDocPtr stylesheetDoc;
	xsltStylesheetPtr stylesheet;
	PyObject * pyStylesheet;
	PyObject * messageHandler;
	PyObject * method;


	if (! PyArg_ParseTuple(args, "ssO", &stylesheetType, &stylesheetSource,
		&messageHandler))
	{
		PyErr_SetString(PyExc_TypeError, "usage: string string {object|None}");
		return NULL;
	} /* if */

	if (messageHandler != Py_None)
	{
		if (! PyInstance_Check(messageHandler))
		{
			PyErr_SetString(PyExc_TypeError, "message handler must be and instance of a class");
			return NULL;
		} /* if */

		method = PyObject_GetAttrString(messageHandler, "write");
		if (method == NULL)
		{
			PyErr_SetString(PyExc_TypeError, "message handler must have a 'write' method");
			return NULL;
		} /* if */
	} /* if */

	xmlSubstituteEntitiesDefault(1);
	xmlLoadExtDtdDefaultValue = 1;

	stylesheetDoc = NULL;
	switch (stylesheetType[0])
	{
	case 'f':
		stylesheetDoc = SAXParseFile(
			(const xmlChar *)stylesheetSource, 0, messageHandler);
		break;
	case 's':
		stylesheetDoc = SAXParseMemory(
			(xmlChar *)stylesheetSource,
			strlen(stylesheetSource), 0, messageHandler);
		break;
	default:
		break;
	} /* switch */
	
	if (stylesheetDoc == NULL)
	{
		PyErr_SetString(PyExc_RuntimeError, "Can't parse stylesheet");
		return NULL;
	} /* if */

	stylesheet = xsltParseStylesheetDoc(stylesheetDoc);

	pyStylesheet = (PyObject *)newLibxslt_stylesheetObject(stylesheet);
	return pyStylesheet;
} /* compile_stylesheet */

/* ------------------------------------------- */

static PyMethodDef libxsltmod_Methods[] = {
	{"translate_to_file", translate_to_file, METH_VARARGS}, 
	{"translate_to_string", translate_to_string, METH_VARARGS}, 
	{"translate_to_stream", translate_to_stream, METH_VARARGS}, 
	{"compile_stylesheet", compile_stylesheet, METH_VARARGS}, 
	{NULL, NULL}
};

/* ------------------------------------------- */

void initlibxsltmod(void)
{
	Py_InitModule("libxsltmod", libxsltmod_Methods);
} /* initlibxsltmod */

/* ------------------------------------------- */

